@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2023 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -146,6 +146,9 @@ const alg_kind_t softmax_log = dnnl_softmax_log;
 using data_type_t = dnnl_data_type_t;
 namespace data_type {
 const data_type_t undef = dnnl_data_type_undef;
+const data_type_t e8m0 = dnnl_e8m0;
+const data_type_t f8_e5m2 = dnnl_f8_e5m2;
+const data_type_t f8_e4m3 = dnnl_f8_e4m3;
 const data_type_t f16 = dnnl_f16;
 const data_type_t bf16 = dnnl_bf16;
 const data_type_t f32 = dnnl_f32;
@@ -153,6 +156,8 @@ const data_type_t f64 = dnnl_f64;
 const data_type_t s32 = dnnl_s32;
 const data_type_t s8 = dnnl_s8;
 const data_type_t u8 = dnnl_u8;
+const data_type_t s4 = dnnl_s4;
+const data_type_t u4 = dnnl_u4;
 const data_type_t boolean = dnnl_boolean;
 const data_type_t data_type_max = dnnl_data_type_max;
 
@@ -170,17 +175,35 @@ const fpmath_mode_t tf32 = dnnl_fpmath_mode_tf32;
 const fpmath_mode_t any = dnnl_fpmath_mode_any;
 } // namespace fpmath_mode
 
+using accumulation_mode_t = dnnl_accumulation_mode_t;
+namespace accumulation_mode {
+const accumulation_mode_t strict = dnnl_accumulation_mode_strict;
+const accumulation_mode_t relaxed = dnnl_accumulation_mode_relaxed;
+const accumulation_mode_t any = dnnl_accumulation_mode_any;
+const accumulation_mode_t f32 = dnnl_accumulation_mode_f32;
+const accumulation_mode_t s32 = dnnl_accumulation_mode_s32;
+const accumulation_mode_t f16 = dnnl_accumulation_mode_f16;
+} // namespace accumulation_mode
+
 using scratchpad_mode_t = dnnl_scratchpad_mode_t;
 namespace scratchpad_mode {
 const scratchpad_mode_t library = dnnl_scratchpad_mode_library;
 const scratchpad_mode_t user = dnnl_scratchpad_mode_user;
 } // namespace scratchpad_mode
 
+using rounding_mode_t = dnnl_rounding_mode_t;
+namespace rounding_mode {
+const rounding_mode_t environment = dnnl_rounding_mode_environment;
+const rounding_mode_t stochastic = dnnl_rounding_mode_stochastic;
+} // namespace rounding_mode
+
 #ifdef DNNL_EXPERIMENTAL_SPARSE
 using sparse_encoding_t = dnnl_sparse_encoding_t;
 namespace sparse_encoding {
 const sparse_encoding_t undef = dnnl_sparse_encoding_undef;
 const sparse_encoding_t csr = dnnl_csr;
+const sparse_encoding_t coo = dnnl_coo;
+const sparse_encoding_t packed = dnnl_packed;
 } // namespace sparse_encoding
 #else
 // Declare dummy values to avoid guarding internal implementation.
@@ -188,6 +211,8 @@ using sparse_encoding_t = int;
 namespace sparse_encoding {
 const sparse_encoding_t undef = 0;
 const sparse_encoding_t csr = 1;
+const sparse_encoding_t packed = 2;
+const sparse_encoding_t coo = 3;
 } // namespace sparse_encoding
 #endif
 
@@ -200,13 +225,14 @@ const format_kind_t opaque = dnnl_format_kind_opaque;
 #ifdef DNNL_EXPERIMENTAL_SPARSE
 const format_kind_t sparse = dnnl_format_kind_sparse;
 #else
-const format_kind_t sparse = dnnl_format_kind_undef;
+const format_kind_t sparse = static_cast<format_kind_t>(4);
 #endif
 
 // Internal only format kinds.
 const format_kind_t internal_only_start = (format_kind_t)(1 << 8);
 const format_kind_t wino = internal_only_start;
 const format_kind_t rnn_packed = (format_kind_t)(internal_only_start + 1);
+const format_kind_t cublaslt_blocked = (format_kind_t)(internal_only_start + 2);
 } // namespace format_kind
 
 #ifdef DNNL_EXPERIMENTAL_PROFILING
@@ -266,13 +292,16 @@ const format_tag_t ba = dnnl_ba;
 const format_tag_t bac = dnnl_bac;
 const format_tag_t bacd = dnnl_bacd;
 const format_tag_t bca = dnnl_bca;
+const format_tag_t bcad = dnnl_bcad;
 const format_tag_t bcda = dnnl_bcda;
 const format_tag_t bcdea = dnnl_bcdea;
 const format_tag_t bacde = dnnl_bacde;
 const format_tag_t cab = dnnl_cab;
 const format_tag_t cba = dnnl_cba;
+const format_tag_t cabd = dnnl_cabd;
 const format_tag_t cdab = dnnl_cdab;
 const format_tag_t cdba = dnnl_cdba;
+const format_tag_t dabc = dnnl_dabc;
 const format_tag_t dcab = dnnl_dcab;
 const format_tag_t cdeab = dnnl_cdeab;
 const format_tag_t cdeba = dnnl_cdeba;
@@ -314,6 +343,8 @@ const format_tag_t AB16b16a2b = dnnl_AB16b16a2b;
 const format_tag_t AB16b32a2b = dnnl_AB16b32a2b;
 const format_tag_t AB16b48a2b = dnnl_AB16b48a2b;
 const format_tag_t AB16b64a2b = dnnl_AB16b64a2b;
+const format_tag_t BA4b4a = dnnl_BA4b4a;
+const format_tag_t BA8b4a = dnnl_BA8b4a;
 const format_tag_t BA16a16b = dnnl_BA16a16b;
 const format_tag_t BA16a32b = dnnl_BA16a32b;
 const format_tag_t BA16a48b = dnnl_BA16a48b;
@@ -339,6 +370,9 @@ const format_tag_t aCB16b32c4b = dnnl_aCB16b32c4b;
 const format_tag_t aCB16b48c4b = dnnl_aCB16b48c4b;
 const format_tag_t aCB16b64c4b = dnnl_aCB16b64c4b;
 
+const format_tag_t Ab4a = dnnl_Ab4a;
+const format_tag_t Ab8a = dnnl_Ab8a;
+const format_tag_t Ab32a = dnnl_Ab32a;
 const format_tag_t Abc16a = dnnl_Abc16a;
 const format_tag_t ABc16a16b = dnnl_ABc16a16b;
 const format_tag_t ABc4a2b = dnnl_ABc4a2b;
@@ -346,16 +380,25 @@ const format_tag_t ABc4a4b = dnnl_ABc4a4b;
 const format_tag_t aBc16b = dnnl_aBc16b;
 const format_tag_t aBc32b = dnnl_aBc32b;
 const format_tag_t ABc16b16a = dnnl_ABc16b16a;
+const format_tag_t AcB16b16a = dnnl_AcB16b16a;
 const format_tag_t ABc16b32a = dnnl_ABc16b32a;
+const format_tag_t AcB16b32a = dnnl_AcB16b32a;
 const format_tag_t ABc16b48a = dnnl_ABc16b48a;
+const format_tag_t AcB16b48a = dnnl_AcB16b48a;
 const format_tag_t ABc16b64a = dnnl_ABc16b64a;
+const format_tag_t AcB16b64a = dnnl_AcB16b64a;
 const format_tag_t Abc4a = dnnl_Abc4a;
 const format_tag_t aBc4b = dnnl_aBc4b;
 const format_tag_t ABc4b8a4b = dnnl_ABc4b8a4b;
+const format_tag_t AcB4b8a4b = dnnl_AcB4b8a4b;
 const format_tag_t ABc4b16a4b = dnnl_ABc4b16a4b;
+const format_tag_t AcB4b16a4b = dnnl_AcB4b16a4b;
 const format_tag_t ABc4b24a4b = dnnl_ABc4b24a4b;
+const format_tag_t AcB4b24a4b = dnnl_AcB4b24a4b;
 const format_tag_t ABc4b32a4b = dnnl_ABc4b32a4b;
+const format_tag_t AcB4b32a4b = dnnl_AcB4b32a4b;
 const format_tag_t ABc4b64a4b = dnnl_ABc4b64a4b;
+const format_tag_t AcB4b64a4b = dnnl_AcB4b64a4b;
 const format_tag_t ABc2b8a4b = dnnl_ABc2b8a4b;
 const format_tag_t ABc16b16a4b = dnnl_ABc16b16a4b;
 const format_tag_t ABc16b32a4b = dnnl_ABc16b32a4b;
@@ -374,9 +417,13 @@ const format_tag_t ABc8a2b = dnnl_ABc8a2b;
 const format_tag_t ABc8a4b = dnnl_ABc8a4b;
 const format_tag_t aBc8b = dnnl_aBc8b;
 const format_tag_t ABc8b16a2b = dnnl_ABc8b16a2b;
+const format_tag_t AcB8b16a2b = dnnl_AcB8b16a2b;
 const format_tag_t ABc8b32a2b = dnnl_ABc8b32a2b;
+const format_tag_t AcB8b32a2b = dnnl_AcB8b32a2b;
 const format_tag_t ABc8b64a2b = dnnl_ABc8b64a2b;
+const format_tag_t AcB8b64a2b = dnnl_AcB8b64a2b;
 const format_tag_t ABc8b8a = dnnl_ABc8b8a;
+const format_tag_t AcB8b8a = dnnl_AcB8b8a;
 const format_tag_t Abcd16a = dnnl_Abcd16a;
 const format_tag_t Abcd8a = dnnl_Abcd8a;
 const format_tag_t Abcd32a = dnnl_Abcd32a;
@@ -384,18 +431,27 @@ const format_tag_t ABcd16a16b = dnnl_ABcd16a16b;
 const format_tag_t aBcd16b = dnnl_aBcd16b;
 const format_tag_t aBcd32b = dnnl_aBcd32b;
 const format_tag_t ABcd16b16a = dnnl_ABcd16b16a;
+const format_tag_t AcdB16b16a = dnnl_AcdB16b16a;
 const format_tag_t ABcd16b32a = dnnl_ABcd16b32a;
+const format_tag_t AcdB16b32a = dnnl_AcdB16b32a;
 const format_tag_t ABcd16b48a = dnnl_ABcd16b48a;
+const format_tag_t AcdB16b48a = dnnl_AcdB16b48a;
 const format_tag_t ABcd16b64a = dnnl_ABcd16b64a;
+const format_tag_t AcdB16b64a = dnnl_AcdB16b64a;
 const format_tag_t aBCd16b16c = dnnl_aBCd16b16c;
 const format_tag_t aBCd16c16b = dnnl_aBCd16c16b;
 const format_tag_t Abcd4a = dnnl_Abcd4a;
 const format_tag_t aBcd4b = dnnl_aBcd4b;
 const format_tag_t ABcd4b8a4b = dnnl_ABcd4b8a4b;
+const format_tag_t AcdB4b8a4b = dnnl_AcdB4b8a4b;
 const format_tag_t ABcd4b16a4b = dnnl_ABcd4b16a4b;
+const format_tag_t AcdB4b16a4b = dnnl_AcdB4b16a4b;
 const format_tag_t ABcd4b24a4b = dnnl_ABcd4b24a4b;
+const format_tag_t AcdB4b24a4b = dnnl_AcdB4b24a4b;
 const format_tag_t ABcd4b32a4b = dnnl_ABcd4b32a4b;
+const format_tag_t AcdB4b32a4b = dnnl_AcdB4b32a4b;
 const format_tag_t ABcd4b64a4b = dnnl_ABcd4b64a4b;
+const format_tag_t AcdB4b64a4b = dnnl_AcdB4b64a4b;
 const format_tag_t ABcd16b16a4b = dnnl_ABcd16b16a4b;
 const format_tag_t ABcd16b32a4b = dnnl_ABcd16b32a4b;
 const format_tag_t ABcd16b48a4b = dnnl_ABcd16b48a4b;
@@ -423,13 +479,17 @@ const format_tag_t ABcd8a4b = dnnl_ABcd8a4b;
 const format_tag_t ABcd8a2b = dnnl_ABcd8a2b;
 const format_tag_t aBcd8b = dnnl_aBcd8b;
 const format_tag_t ABcd8b16a2b = dnnl_ABcd8b16a2b;
+const format_tag_t AcdB8b16a2b = dnnl_AcdB8b16a2b;
 const format_tag_t ABcd8b32a2b = dnnl_ABcd8b32a2b;
+const format_tag_t AcdB8b32a2b = dnnl_AcdB8b32a2b;
 const format_tag_t ABcd8b64a2b = dnnl_ABcd8b64a2b;
+const format_tag_t AcdB8b64a2b = dnnl_AcdB8b64a2b;
 const format_tag_t ABcd2b8a4b = dnnl_ABcd2b8a4b;
 const format_tag_t aBCd8b16c2b = dnnl_aBCd8b16c2b;
 const format_tag_t aCBd8b16c2b = dnnl_aCBd8b16c2b;
 const format_tag_t aBCd2c8b16c2b = dnnl_aBCd2c8b16c2b;
 const format_tag_t ABcd8b8a = dnnl_ABcd8b8a;
+const format_tag_t AcdB8b8a = dnnl_AcdB8b8a;
 const format_tag_t aBCd8b8c = dnnl_aBCd8b8c;
 const format_tag_t aBCd8b2c = dnnl_aBCd8b2c;
 const format_tag_t aBCd8b4c = dnnl_aBCd8b4c;
@@ -441,9 +501,13 @@ const format_tag_t ABcde16a16b = dnnl_ABcde16a16b;
 const format_tag_t aBcde16b = dnnl_aBcde16b;
 const format_tag_t aBcde32b = dnnl_aBcde32b;
 const format_tag_t ABcde16b16a = dnnl_ABcde16b16a;
+const format_tag_t AcdeB16b16a = dnnl_AcdeB16b16a;
 const format_tag_t ABcde16b32a = dnnl_ABcde16b32a;
+const format_tag_t AcdeB16b32a = dnnl_AcdeB16b32a;
 const format_tag_t ABcde16b48a = dnnl_ABcde16b48a;
+const format_tag_t AcdeB16b48a = dnnl_AcdeB16b48a;
 const format_tag_t ABcde16b64a = dnnl_ABcde16b64a;
+const format_tag_t AcdeB16b64a = dnnl_AcdeB16b64a;
 const format_tag_t aBCde16b16c = dnnl_aBCde16b16c;
 const format_tag_t aBCde16c16b = dnnl_aBCde16c16b;
 const format_tag_t aBCde2c8b4c = dnnl_aBCde2c8b4c;
@@ -464,15 +528,23 @@ const format_tag_t ABcde8a2b = dnnl_ABcde8a2b;
 const format_tag_t ABcde8a4b = dnnl_ABcde8a4b;
 const format_tag_t aBcde8b = dnnl_aBcde8b;
 const format_tag_t ABcde8b16a2b = dnnl_ABcde8b16a2b;
+const format_tag_t AcdeB8b16a2b = dnnl_AcdeB8b16a2b;
 const format_tag_t ABcde8b32a2b = dnnl_ABcde8b32a2b;
+const format_tag_t AcdeB8b32a2b = dnnl_AcdeB8b32a2b;
 const format_tag_t ABcde8b64a2b = dnnl_ABcde8b64a2b;
+const format_tag_t AcdeB8b64a2b = dnnl_AcdeB8b64a2b;
 const format_tag_t ABcde8a16b2a = dnnl_ABcde8a16b2a;
 const format_tag_t BAcde8a16b2a = dnnl_BAcde8a16b2a;
 const format_tag_t ABcde4b8a4b = dnnl_ABcde4b8a4b;
+const format_tag_t AcdeB4b8a4b = dnnl_AcdeB4b8a4b;
 const format_tag_t ABcde4b16a4b = dnnl_ABcde4b16a4b;
+const format_tag_t AcdeB4b16a4b = dnnl_AcdeB4b16a4b;
 const format_tag_t ABcde4b24a4b = dnnl_ABcde4b24a4b;
+const format_tag_t AcdeB4b24a4b = dnnl_AcdeB4b24a4b;
 const format_tag_t ABcde4b32a4b = dnnl_ABcde4b32a4b;
+const format_tag_t AcdeB4b32a4b = dnnl_AcdeB4b32a4b;
 const format_tag_t ABcde4b64a4b = dnnl_ABcde4b64a4b;
+const format_tag_t AcdeB4b64a4b = dnnl_AcdeB4b64a4b;
 const format_tag_t ABcde16b16a4b = dnnl_ABcde16b16a4b;
 const format_tag_t ABcde16b32a4b = dnnl_ABcde16b32a4b;
 const format_tag_t ABcde16b48a4b = dnnl_ABcde16b48a4b;
@@ -481,6 +553,7 @@ const format_tag_t ABcde2b8a4b = dnnl_ABcde2b8a4b;
 const format_tag_t aBCde8b16c2b = dnnl_aBCde8b16c2b;
 const format_tag_t aCBde8b16c2b = dnnl_aCBde8b16c2b;
 const format_tag_t ABcde8b8a = dnnl_ABcde8b8a;
+const format_tag_t AcdeB8b8a = dnnl_AcdeB8b8a;
 const format_tag_t aBCde8b8c = dnnl_aBCde8b8c;
 const format_tag_t aBCde8b2c = dnnl_aBCde8b2c;
 const format_tag_t aBCde8b4c = dnnl_aBCde8b4c;
@@ -940,16 +1013,25 @@ const format_tag_t aBdeC24b2c = dnnl_aBdeC24b2c;
 const format_tag_t aBdefC24b2c = dnnl_aBdefC24b2c;
 const format_tag_t AB8b32a = dnnl_AB8b32a;
 const format_tag_t ABc8b32a = dnnl_ABc8b32a;
+const format_tag_t AcB8b32a = dnnl_AcB8b32a;
 const format_tag_t ABcd8b32a = dnnl_ABcd8b32a;
+const format_tag_t AcdB8b32a = dnnl_AcdB8b32a;
 const format_tag_t ABcde8b32a = dnnl_ABcde8b32a;
+const format_tag_t AcdeB8b32a = dnnl_AcdeB8b32a;
 const format_tag_t AB8b24a = dnnl_AB8b24a;
 const format_tag_t ABc8b24a = dnnl_ABc8b24a;
+const format_tag_t AcB8b24a = dnnl_AcB8b24a;
 const format_tag_t ABcd8b24a = dnnl_ABcd8b24a;
+const format_tag_t AcdB8b24a = dnnl_AcdB8b24a;
 const format_tag_t ABcde8b24a = dnnl_ABcde8b24a;
+const format_tag_t AcdeB8b24a = dnnl_AcdeB8b24a;
 const format_tag_t AB8b16a = dnnl_AB8b16a;
 const format_tag_t ABc8b16a = dnnl_ABc8b16a;
+const format_tag_t AcB8b16a = dnnl_AcB8b16a;
 const format_tag_t ABcd8b16a = dnnl_ABcd8b16a;
+const format_tag_t AcdB8b16a = dnnl_AcdB8b16a;
 const format_tag_t ABcde8b16a = dnnl_ABcde8b16a;
+const format_tag_t AcdeB8b16a = dnnl_AcdeB8b16a;
 const format_tag_t AB8b8a = dnnl_AB8b8a;
 const format_tag_t AcB24a4b = dnnl_AcB24a4b;
 const format_tag_t AcdB24a4b = dnnl_AcdB24a4b;
@@ -959,12 +1041,28 @@ const format_tag_t aBdeC24b4c = dnnl_aBdeC24b4c;
 const format_tag_t aBdefC24b4c = dnnl_aBdefC24b4c;
 const format_tag_t AB8b8a2b = dnnl_AB8b8a2b;
 const format_tag_t ABc8b8a2b = dnnl_ABc8b8a2b;
+const format_tag_t AcB8b8a2b = dnnl_AcB8b8a2b;
 const format_tag_t ABcd8b8a2b = dnnl_ABcd8b8a2b;
+const format_tag_t AcdB8b8a2b = dnnl_AcdB8b8a2b;
 const format_tag_t ABcde8b8a2b = dnnl_ABcde8b8a2b;
+const format_tag_t AcdeB8b8a2b = dnnl_AcdeB8b8a2b;
 const format_tag_t AB8b24a2b = dnnl_AB8b24a2b;
 const format_tag_t ABc8b24a2b = dnnl_ABc8b24a2b;
+const format_tag_t AcB8b24a2b = dnnl_AcB8b24a2b;
 const format_tag_t ABcd8b24a2b = dnnl_ABcd8b24a2b;
+const format_tag_t AcdB8b24a2b = dnnl_AcdB8b24a2b;
 const format_tag_t ABcde8b24a2b = dnnl_ABcde8b24a2b;
+const format_tag_t AcdeB8b24a2b = dnnl_AcdeB8b24a2b;
+const format_tag_t BA2a24b = dnnl_BA2a24b;
+const format_tag_t aCB2b24c = dnnl_aCB2b24c;
+const format_tag_t BA2a8b = dnnl_BA2a8b;
+const format_tag_t aCB2b8c = dnnl_aCB2b8c;
+const format_tag_t BA8a24b = dnnl_BA8a24b;
+const format_tag_t aCB8b24c = dnnl_aCB8b24c;
+const format_tag_t BA8a16b = dnnl_BA8a16b;
+const format_tag_t aCB8b16c = dnnl_aCB8b16c;
+const format_tag_t BA8a8b = dnnl_BA8a8b;
+const format_tag_t aCB8b8c = dnnl_aCB8b8c;
 
 const format_tag_t last = dnnl_format_tag_last;
 
@@ -1074,16 +1172,25 @@ const format_tag_t IOw16o16i = dnnl_IOw16o16i;
 const format_tag_t IOw16i16o = dnnl_IOw16i16o;
 const format_tag_t gIOw16i16o = dnnl_gIOw16i16o;
 const format_tag_t OIw16i16o = dnnl_OIw16i16o;
+const format_tag_t OwI16i16o = dnnl_OwI16i16o;
 const format_tag_t OIw16i32o = dnnl_OIw16i32o;
+const format_tag_t OwI16i32o = dnnl_OwI16i32o;
 const format_tag_t OIw16i48o = dnnl_OIw16i48o;
+const format_tag_t OwI16i48o = dnnl_OwI16i48o;
 const format_tag_t OIw16i64o = dnnl_OIw16i64o;
+const format_tag_t OwI16i64o = dnnl_OwI16i64o;
 const format_tag_t OIw16o16i = dnnl_OIw16o16i;
 const format_tag_t Oiw16o = dnnl_Oiw16o;
 const format_tag_t OIw4i8o4i = dnnl_OIw4i8o4i;
+const format_tag_t OwI4i8o4i = dnnl_OwI4i8o4i;
 const format_tag_t OIw4i16o4i = dnnl_OIw4i16o4i;
+const format_tag_t OwI4i16o4i = dnnl_OwI4i16o4i;
 const format_tag_t OIw4i24o4i = dnnl_OIw4i24o4i;
+const format_tag_t OwI4i24o4i = dnnl_OwI4i24o4i;
 const format_tag_t OIw4i32o4i = dnnl_OIw4i32o4i;
+const format_tag_t OwI4i32o4i = dnnl_OwI4i32o4i;
 const format_tag_t OIw4i64o4i = dnnl_OIw4i64o4i;
+const format_tag_t OwI4i64o4i = dnnl_OwI4i64o4i;
 const format_tag_t OIw2i8o4i = dnnl_OIw2i8o4i;
 const format_tag_t OIw16i16o4i = dnnl_OIw16i16o4i;
 const format_tag_t OIw16i32o4i = dnnl_OIw16i32o4i;
@@ -1098,9 +1205,13 @@ const format_tag_t OIw4i4o = dnnl_OIw4i4o;
 const format_tag_t OIw4o4i = dnnl_OIw4o4i;
 const format_tag_t Oiw4o = dnnl_Oiw4o;
 const format_tag_t OIw8i16o2i = dnnl_OIw8i16o2i;
+const format_tag_t OwI8i16o2i = dnnl_OwI8i16o2i;
 const format_tag_t OIw8i32o2i = dnnl_OIw8i32o2i;
+const format_tag_t OwI8i32o2i = dnnl_OwI8i32o2i;
 const format_tag_t OIw8i64o2i = dnnl_OIw8i64o2i;
+const format_tag_t OwI8i64o2i = dnnl_OwI8i64o2i;
 const format_tag_t OIw8i8o = dnnl_OIw8i8o;
+const format_tag_t OwI8i8o = dnnl_OwI8i8o;
 const format_tag_t OIw8o16i2o = dnnl_OIw8o16i2o;
 const format_tag_t IOw8o16i2o = dnnl_IOw8o16i2o;
 const format_tag_t OIw8o8i = dnnl_OIw8o8i;
@@ -1122,16 +1233,25 @@ const format_tag_t Ohwi8o = dnnl_Ohwi8o;
 const format_tag_t OhwI8o2i = dnnl_OhwI8o2i;
 const format_tag_t OhwI8o4i = dnnl_OhwI8o4i;
 const format_tag_t OIhw16i16o = dnnl_OIhw16i16o;
+const format_tag_t OhwI16i16o = dnnl_OhwI16i16o;
 const format_tag_t OIhw16i32o = dnnl_OIhw16i32o;
+const format_tag_t OhwI16i32o = dnnl_OhwI16i32o;
 const format_tag_t OIhw16i48o = dnnl_OIhw16i48o;
+const format_tag_t OhwI16i48o = dnnl_OhwI16i48o;
 const format_tag_t OIhw16i64o = dnnl_OIhw16i64o;
+const format_tag_t OhwI16i64o = dnnl_OhwI16i64o;
 const format_tag_t OIhw16o16i = dnnl_OIhw16o16i;
 const format_tag_t Oihw16o = dnnl_Oihw16o;
 const format_tag_t OIhw4i8o4i = dnnl_OIhw4i8o4i;
+const format_tag_t OhwI4i8o4i = dnnl_OhwI4i8o4i;
 const format_tag_t OIhw4i16o4i = dnnl_OIhw4i16o4i;
+const format_tag_t OhwI4i16o4i = dnnl_OhwI4i16o4i;
 const format_tag_t OIhw4i24o4i = dnnl_OIhw4i24o4i;
+const format_tag_t OhwI4i24o4i = dnnl_OhwI4i24o4i;
 const format_tag_t OIhw4i32o4i = dnnl_OIhw4i32o4i;
+const format_tag_t OhwI4i32o4i = dnnl_OhwI4i32o4i;
 const format_tag_t OIhw4i64o4i = dnnl_OIhw4i64o4i;
+const format_tag_t OhwI4i64o4i = dnnl_OhwI4i64o4i;
 const format_tag_t OIhw16i16o4i = dnnl_OIhw16i16o4i;
 const format_tag_t OIhw16i32o4i = dnnl_OIhw16i32o4i;
 const format_tag_t OIhw16i48o4i = dnnl_OIhw16i48o4i;
@@ -1145,10 +1265,14 @@ const format_tag_t OIhw4i4o = dnnl_OIhw4i4o;
 const format_tag_t OIhw4o4i = dnnl_OIhw4o4i;
 const format_tag_t Oihw4o = dnnl_Oihw4o;
 const format_tag_t OIhw8i16o2i = dnnl_OIhw8i16o2i;
+const format_tag_t OhwI8i16o2i = dnnl_OhwI8i16o2i;
 const format_tag_t OIhw8i32o2i = dnnl_OIhw8i32o2i;
+const format_tag_t OhwI8i32o2i = dnnl_OhwI8i32o2i;
 const format_tag_t OIhw8i64o2i = dnnl_OIhw8i64o2i;
+const format_tag_t OhwI8i64o2i = dnnl_OhwI8i64o2i;
 const format_tag_t OIhw2i8o4i = dnnl_OIhw2i8o4i;
 const format_tag_t OIhw8i8o = dnnl_OIhw8i8o;
+const format_tag_t OhwI8i8o = dnnl_OhwI8i8o;
 const format_tag_t OIhw8o16i2o = dnnl_OIhw8o16i2o;
 const format_tag_t IOhw8o16i2o = dnnl_IOhw8o16i2o;
 const format_tag_t OIhw8o8i = dnnl_OIhw8o8i;
@@ -1163,9 +1287,13 @@ const format_tag_t Odhwi8o = dnnl_Odhwi8o;
 const format_tag_t OdhwI8o2i = dnnl_OdhwI8o2i;
 const format_tag_t OdhwI8o4i = dnnl_OdhwI8o4i;
 const format_tag_t OIdhw16i16o = dnnl_OIdhw16i16o;
+const format_tag_t OdhwI16i16o = dnnl_OdhwI16i16o;
 const format_tag_t OIdhw16i32o = dnnl_OIdhw16i32o;
+const format_tag_t OdhwI16i32o = dnnl_OdhwI16i32o;
 const format_tag_t OIdhw16i48o = dnnl_OIdhw16i48o;
+const format_tag_t OdhwI16i48o = dnnl_OdhwI16i48o;
 const format_tag_t OIdhw16i64o = dnnl_OIdhw16i64o;
+const format_tag_t OdhwI16i64o = dnnl_OdhwI16i64o;
 const format_tag_t OIdhw16o16i = dnnl_OIdhw16o16i;
 const format_tag_t OIdhw16o16i2o = dnnl_OIdhw16o16i2o;
 const format_tag_t Oidhw16o = dnnl_Oidhw16o;
@@ -1173,13 +1301,21 @@ const format_tag_t OIdhw4i4o = dnnl_OIdhw4i4o;
 const format_tag_t OIdhw4o4i = dnnl_OIdhw4o4i;
 const format_tag_t Oidhw4o = dnnl_Oidhw4o;
 const format_tag_t OIdhw8i16o2i = dnnl_OIdhw8i16o2i;
+const format_tag_t OdhwI8i16o2i = dnnl_OdhwI8i16o2i;
 const format_tag_t OIdhw8i32o2i = dnnl_OIdhw8i32o2i;
+const format_tag_t OdhwI8i32o2i = dnnl_OdhwI8i32o2i;
 const format_tag_t OIdhw8i64o2i = dnnl_OIdhw8i64o2i;
+const format_tag_t OdhwI8i64o2i = dnnl_OdhwI8i64o2i;
 const format_tag_t OIdhw4i8o4i = dnnl_OIdhw4i8o4i;
+const format_tag_t OdhwI4i8o4i = dnnl_OdhwI4i8o4i;
 const format_tag_t OIdhw4i16o4i = dnnl_OIdhw4i16o4i;
+const format_tag_t OdhwI4i16o4i = dnnl_OdhwI4i16o4i;
 const format_tag_t OIdhw4i24o4i = dnnl_OIdhw4i24o4i;
+const format_tag_t OdhwI4i24o4i = dnnl_OdhwI4i24o4i;
 const format_tag_t OIdhw4i32o4i = dnnl_OIdhw4i32o4i;
+const format_tag_t OdhwI4i32o4i = dnnl_OdhwI4i32o4i;
 const format_tag_t OIdhw4i64o4i = dnnl_OIdhw4i64o4i;
+const format_tag_t OdhwI4i64o4i = dnnl_OdhwI4i64o4i;
 const format_tag_t OIdhw16i16o4i = dnnl_OIdhw16i16o4i;
 const format_tag_t OIdhw16i32o4i = dnnl_OIdhw16i32o4i;
 const format_tag_t OIdhw16i48o4i = dnnl_OIdhw16i48o4i;
@@ -1192,6 +1328,7 @@ const format_tag_t OIdhw2i8o4i = dnnl_OIdhw2i8o4i;
 const format_tag_t OIdhw8o16i2o = dnnl_OIdhw8o16i2o;
 const format_tag_t IOdhw8o16i2o = dnnl_IOdhw8o16i2o;
 const format_tag_t OIdhw8i8o = dnnl_OIdhw8i8o;
+const format_tag_t OdhwI8i8o = dnnl_OdhwI8i8o;
 const format_tag_t OIdhw8o8i = dnnl_OIdhw8o8i;
 const format_tag_t OIdhw8o4i = dnnl_OIdhw8o4i;
 const format_tag_t gIOw16o16i = dnnl_gIOw16o16i;
@@ -1708,25 +1845,40 @@ const format_tag_t gOdhwI24o4i = dnnl_gOdhwI24o4i;
 
 const format_tag_t OI8i32o = dnnl_OI8i32o;
 const format_tag_t OIw8i32o = dnnl_OIw8i32o;
+const format_tag_t OwI8i32o = dnnl_OwI8i32o;
 const format_tag_t OIhw8i32o = dnnl_OIhw8i32o;
+const format_tag_t OhwI8i32o = dnnl_OhwI8i32o;
 const format_tag_t OIdhw8i32o = dnnl_OIdhw8i32o;
+const format_tag_t OdhwI8i32o = dnnl_OdhwI8i32o;
 const format_tag_t OI8i24o = dnnl_OI8i24o;
 const format_tag_t OIw8i24o = dnnl_OIw8i24o;
+const format_tag_t OwI8i24o = dnnl_OwI8i24o;
 const format_tag_t OIhw8i24o = dnnl_OIhw8i24o;
+const format_tag_t OhwI8i24o = dnnl_OhwI8i24o;
 const format_tag_t OIdhw8i24o = dnnl_OIdhw8i24o;
+const format_tag_t OdhwI8i24o = dnnl_OdhwI8i24o;
 const format_tag_t OI8i16o = dnnl_OI8i16o;
 const format_tag_t OIw8i16o = dnnl_OIw8i16o;
+const format_tag_t OwI8i16o = dnnl_OwI8i16o;
 const format_tag_t OIhw8i16o = dnnl_OIhw8i16o;
+const format_tag_t OhwI8i16o = dnnl_OhwI8i16o;
 const format_tag_t OIdhw8i16o = dnnl_OIdhw8i16o;
+const format_tag_t OdhwI8i16o = dnnl_OdhwI8i16o;
 const format_tag_t OI8i8o = dnnl_OI8i8o;
 const format_tag_t OI8i8o2i = dnnl_OI8i8o2i;
 const format_tag_t OIw8i8o2i = dnnl_OIw8i8o2i;
+const format_tag_t OwI8i8o2i = dnnl_OwI8i8o2i;
 const format_tag_t OIhw8i8o2i = dnnl_OIhw8i8o2i;
+const format_tag_t OhwI8i8o2i = dnnl_OhwI8i8o2i;
 const format_tag_t OIdhw8i8o2i = dnnl_OIdhw8i8o2i;
+const format_tag_t OdhwI8i8o2i = dnnl_OdhwI8i8o2i;
 const format_tag_t OI8i24o2i = dnnl_OI8i24o2i;
 const format_tag_t OIw8i24o2i = dnnl_OIw8i24o2i;
+const format_tag_t OwI8i24o2i = dnnl_OwI8i24o2i;
 const format_tag_t OIhw8i24o2i = dnnl_OIhw8i24o2i;
+const format_tag_t OhwI8i24o2i = dnnl_OhwI8i24o2i;
 const format_tag_t OIdhw8i24o2i = dnnl_OIdhw8i24o2i;
+const format_tag_t OdhwI8i24o2i = dnnl_OdhwI8i24o2i;
 } // namespace format_tag
 
 using normalization_flags_t = dnnl_normalization_flags_t;
@@ -1801,6 +1953,7 @@ const primitive_kind_t group_normalization = dnnl_group_normalization;
 // Internal only primitive kinds.
 const primitive_kind_t internal_only_start = (primitive_kind_t)(1 << 12);
 const primitive_kind_t zero_pad = internal_only_start;
+const primitive_kind_t sdpa = (primitive_kind_t)(internal_only_start + 1);
 } // namespace primitive_kind
 
 using query_t = dnnl_query_t;
@@ -1876,9 +2029,9 @@ const query_t sparse_encoding = dnnl_query_sparse_encoding;
 const query_t nnz_s64 = dnnl_query_nnz_s64;
 const query_t num_handles_s32 = dnnl_query_num_handles_s32;
 #else
-const query_t sparse_encoding = dnnl_query_undef;
-const query_t nnz_s64 = dnnl_query_undef;
-const query_t num_handles_s32 = dnnl_query_undef;
+const query_t sparse_encoding = static_cast<query_t>(266);
+const query_t nnz_s64 = static_cast<query_t>(267);
+const query_t num_handles_s32 = static_cast<query_t>(268);
 #endif
 
 // Internal only query kinds.
