@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2023 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -28,19 +28,20 @@ namespace primitive_hashing {
 
 key_t::key_t(const engine_t *engine, const op_desc_t *op_desc,
         const primitive_attr_t *attr, int pd_iterator_offset,
-        const std::vector<memory_desc_t> &hint_mds)
+        const std::vector<memory_desc_t> &hint_mds, int skip_idx)
     : primitive_kind_(op_desc->kind)
     , op_desc_(op_desc)
     , attr_(attr)
     , pd_iterator_offset_(pd_iterator_offset)
     , impl_nthr_(dnnl_get_max_threads())
+    , skip_idx_(skip_idx)
     , hint_mds_(hint_mds)
     , engine_id_(engine->engine_id())
     , thread_id_(std::this_thread::get_id()) {}
 
 key_t::key_t(const primitive_desc_t *pd, const engine_t *engine)
     : key_t(engine, pd->op_desc(), pd->attr(), pd->pd_iterator_offset(),
-            pd->hint_mds(false /* is_hint */)) {}
+            pd->hint_mds(false /* is_hint */), pd->skip_idx()) {}
 
 bool key_t::operator==(const key_t &rhs) const {
     DNNL_SHORT_CIRCUIT_SELF_COMPARISON(rhs);
@@ -52,6 +53,7 @@ bool key_t::operator==(const key_t &rhs) const {
         && hint_mds_.size() == rhs.hint_mds_.size()
         && pd_iterator_offset_ == rhs.pd_iterator_offset_
         && impl_nthr_ == rhs.impl_nthr_
+        && skip_idx_ == rhs.skip_idx_
         && (*attr_) == (*rhs.attr_);
 
     if (!ret) return false;
@@ -66,7 +68,11 @@ bool key_t::operator==(const key_t &rhs) const {
             CASE(batch_normalization)
             CASE(binary)
             CASE(concat)
-            CASE(convolution)
+            // Use a custom comparison function that ignores alg_kind.
+            case primitive_kind::convolution:
+                ret = compare_conv_opdesc(cast_to_desc<convolution_desc_t>(op_desc_),
+                cast_to_desc<convolution_desc_t>(rhs.op_desc_));
+            break;
             CASE(deconvolution)
             CASE(eltwise)
             CASE(gemm)
@@ -81,6 +87,7 @@ bool key_t::operator==(const key_t &rhs) const {
             CASE(reorder)
             CASE(resampling)
             CASE(rnn)
+            CASE(sdpa)
             CASE(shuffle)
             CASE(softmax)
             CASE(sum)
@@ -136,6 +143,13 @@ size_t get_md_hash(const memory_desc_t &md) {
             seed = hash_combine(seed, md.format_desc.wino_desc.adj_scale);
             seed = hash_combine(seed, md.format_desc.wino_desc.size);
             break;
+        case format_kind::cublaslt_blocked:
+            seed = hash_combine(seed,
+                    static_cast<size_t>(md.format_desc.cublaslt_blocked_desc
+                                                .cublaslt_format));
+            seed = hash_combine(
+                    seed, (md.format_desc.cublaslt_blocked_desc.size));
+            break;
         case format_kind::rnn_packed:
             seed = hash_combine(seed,
                     static_cast<size_t>(md.format_desc.rnn_packed_desc.format));
@@ -163,6 +177,8 @@ size_t get_md_hash(const memory_desc_t &md) {
             seed = get_array_hash(seed,
                     md.format_desc.sparse_desc.metadata_types,
                     sparse_desc_t::max_metadata_types);
+            // User cannot initialize `packed_desc` therefore `packed_desc`
+            // is always zero initialized.
             break;
 #endif
         default: assert(!"unknown format_kind");
@@ -197,7 +213,19 @@ size_t get_attr_hash(const primitive_attr_t &attr) {
     // scratchpad_mode
     seed = hash_combine(seed, static_cast<size_t>(attr.scratchpad_mode_));
     // fpmath_mode
-    seed = hash_combine(seed, static_cast<size_t>(attr.fpmath_mode_));
+    seed = hash_combine(seed, static_cast<size_t>(attr.fpmath_.mode_));
+    seed = hash_combine(seed, static_cast<size_t>(attr.fpmath_.apply_to_int_));
+    // deterministic
+    seed = hash_combine(seed, static_cast<size_t>(attr.deterministic_));
+    // acc_mode
+    seed = hash_combine(seed, static_cast<size_t>(attr.acc_mode_));
+    // rounding_mode
+    if (!attr.rounding_mode_.has_default_values()) {
+        for (const auto &e : attr.rounding_mode_.rounding_modes_map_) {
+            seed = hash_combine(seed, e.first);
+            seed = hash_combine(seed, static_cast<size_t>(e.second));
+        }
+    }
 
     if (!attr.output_scales_.has_default_values()) {
         // output_scales: mask
@@ -209,17 +237,33 @@ size_t get_attr_hash(const primitive_attr_t &attr) {
             seed = hash_combine(seed, p.first);
             // scales: mask
             seed = hash_combine(seed, p.second.mask_);
+            // scales: groups
+            const int ndims = p.second.ndims_;
+            seed = hash_combine(seed, ndims);
+            if (ndims > 0)
+                seed = get_array_hash(seed, p.second.group_dims_, ndims);
+            // scales: data type
+            seed = hash_combine(seed, static_cast<size_t>(p.second.data_type_));
         }
     }
     // zero_points
     for (int arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
         if (!attr.zero_points_.has_default_values(arg)) {
+            const auto &zps = attr.zero_points_;
             // zero_points: arg
             seed = hash_combine(seed, arg);
             int mask = 0;
-            attr.zero_points_.get(arg, &mask);
+            zps.get(arg, &mask);
             // zero_points: mask
             seed = hash_combine(seed, mask);
+            // zero points: groups
+            const int ndims = zps.get_groups_ndims(arg);
+            seed = hash_combine(seed, ndims);
+            if (ndims > 0)
+                seed = get_array_hash(seed, zps.get_groups(arg), ndims);
+            // zero points: data type
+            seed = hash_combine(
+                    seed, static_cast<size_t>(zps.get_data_type(arg)));
         }
     // post_ops: entry[:]
     for (int i = 0; i < attr.post_ops_.len(); i++) {
@@ -279,6 +323,10 @@ size_t get_attr_hash(const primitive_attr_t &attr) {
     if (attr.gpu_attr_) {
         seed = hash_combine(seed, attr.gpu_attr_->get_hash());
     }
+    if (!attr.dropout_.has_default_values()) {
+        seed = hash_combine(
+                seed, get_md_hash(attr.dropout_.user_dropout_desc_));
+    }
     // Combined hash for attributes
     return seed;
 }
@@ -340,7 +388,18 @@ size_t get_desc_hash(const convolution_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
-    seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
+
+    // Ignore `alg_kind` to keep hash value consistent for any algorithm.
+    //
+    // Background: when a convolution primitive descriptor is created for
+    // the algorithm `auto` we overwrite `alg_kind` field in `op_desc` when
+    // store it in the primitive descriptor. Because of that, the `op_desc`
+    // stored in the primitive descriptor is different from the one user
+    // passed to oneDNN API. Because of the difference the requested
+    // primitive descriptor cannot be found in the cache if we hash/compare
+    // `alg_kind`.
+    //seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
+
     // Memory descriptors
     seed = hash_combine(seed, get_md_hash(desc.src_desc));
     seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
@@ -357,6 +416,8 @@ size_t get_desc_hash(const convolution_desc_t &desc) {
     seed = get_array_hash(seed, desc.padding[1], DNNL_MAX_NDIMS);
     // Accumulator type
     seed = hash_combine(seed, static_cast<size_t>(desc.accum_data_type));
+    // Internal member
+    seed = hash_combine(seed, static_cast<size_t>(desc.use_inversion));
     // Combined hash for (de-)convolution desc
     return seed;
 }
@@ -680,6 +741,23 @@ size_t get_desc_hash(const zero_pad_desc_t &desc) {
     return seed;
 }
 
+size_t get_desc_hash(const sdpa_desc_t &desc) {
+    size_t seed = 0;
+    // Kinds
+    seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
+    // Memory descriptors
+    seed = hash_combine(seed, get_md_hash(desc.q_desc));
+    seed = hash_combine(seed, get_md_hash(desc.k_desc));
+    seed = hash_combine(seed, get_md_hash(desc.v_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.attn_mask_desc));
+    // Scale type
+    seed = hash_combine(seed, static_cast<size_t>(desc.scale_dt));
+    seed = hash_combine(seed, desc.invert_scale);
+    // Combined hash for sdpa desc
+    return seed;
+}
+
 } // namespace primitive_hashing
 } // namespace impl
 } // namespace dnnl
