@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2023 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +16,7 @@
 *******************************************************************************/
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/dnnl_thread.hpp"
 #include "common/memory.hpp"
 #include "common/nstl.hpp"
@@ -52,9 +53,7 @@ bool tag_is_flat(format_tag_t tag, format_tag_t ncx, format_tag_t nxc, int ic) {
 jit_avx2_conv_fwd_kernel_f32::jit_avx2_conv_fwd_kernel_f32(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, avx2)
-    , jcp(ajcp)
-    , attr_(attr) {
+    : jit_generator(jit_name(), avx2), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -579,13 +578,15 @@ void jit_avx2_conv_fwd_kernel_f32::generate() {
 
     this->postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
         const memory_desc_wrapper &weights_d, const memory_desc_wrapper &dst_d,
         const primitive_attr_t &attr) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(avx)) return status::unimplemented;
     jcp.isa = mayiuse(avx2) ? avx2 : avx;
 
@@ -637,7 +638,8 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     bool kernel_outside_src = false || ext_kw <= jcp.l_pad
             || ext_kw <= jcp.r_pad || ext_kh <= jcp.t_pad || ext_kh <= jcp.b_pad
             || ext_kd <= jcp.f_pad || ext_kd <= jcp.back_pad;
-    if (kernel_outside_src) return status::unimplemented;
+    VDISPATCH_CONV_IC(!kernel_outside_src, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "weights and src size mismatch");
 
     const auto dat_tag_nxc = pick(ndims - 3, nwc, nhwc, ndhwc);
     const auto dat_tag_ncx = pick(ndims - 3, ncw, nchw, ncdhw);
@@ -661,8 +663,8 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
 
     // Disable this kernel on high width 1d object as gemm performs better until
     // optimizations can be made to fix it.
-    if (is_data_layout_nxc && ndims == 3 && jcp.ow > 11 * 1024)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(!(is_data_layout_nxc && ndims == 3 && jcp.ow > 11 * 1024),
+            VERBOSE_IMPL_HEURISTIC_FAIL, "input shape with large width");
 
     jcp.with_bias = cd.bias_desc.format_kind != format_kind::undef;
 
@@ -695,7 +697,8 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     }
 
     if (jcp.with_eltwise || jcp.with_binary)
-        if (!mayiuse(avx2)) return status::unimplemented;
+        VDISPATCH_CONV_IC(mayiuse(avx2), VERBOSE_UNSUPPORTED_FEATURE,
+                "eltwise and binary post-ops not implemented on isa");
 
     using namespace injector;
     static constexpr bool sum_at_pos_0_only = true;
@@ -704,9 +707,9 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(jcp.isa,
             {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
             sum_requires_scale_one, sum_requires_zp_zero));
-    if (!post_ops_ok_) return status::unimplemented;
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
-    bool args_ok = true
+    bool tag_ok = true
             && IMPLICATION(flat,
                     jcp.wei_tag == wei_tag_Oxio
                             && ((tag_is_flat(jcp.src_tag, dat_tag_ncx,
@@ -719,10 +722,13 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
                             && ((jcp.src_tag == dat_tag_nCx8c
                                         && jcp.dst_tag == dat_tag_nCx8c)
                                     || (jcp.src_tag == dat_tag_nxc
-                                            && jcp.dst_tag == dat_tag_nxc)))
-            && jcp.ic <= src_d.padded_dims()[1]
+                                            && jcp.dst_tag == dat_tag_nxc)));
+    VDISPATCH_CONV_IC(tag_ok, VERBOSE_UNSUPPORTED_TAG);
+
+    bool channel_pad_ok = true && jcp.ic <= src_d.padded_dims()[1]
             && jcp.oc <= dst_d.padded_dims()[1];
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(channel_pad_ok, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "i/o and padded channel size mismatch");
 
     jcp.ur_h = 1; /* no code-unrolling by h so far */
     jcp.ur_w = 3;
@@ -760,13 +766,17 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     if (jcp.ow < jcp.ur_w) jcp.ur_w = jcp.ow;
     jcp.ur_w_tail = jcp.ow % jcp.ur_w;
 
-    args_ok = true && IMPLICATION(!is_data_layout_nxc, jcp.oc % simd_w == 0)
-            && jcp.l_pad <= jcp.ur_w
-            && IMPLICATION(jcp.kw > 7,
+    VDISPATCH_CONV_IC(IMPLICATION(!is_data_layout_nxc, jcp.oc % simd_w == 0),
+            VERBOSE_SHAPE_RESTRICTION);
+    VDISPATCH_CONV_IC(jcp.l_pad <= jcp.ur_w, VERBOSE_SHAPE_RESTRICTION);
+    VDISPATCH_CONV_IC(
+            IMPLICATION(jcp.kw > 7,
                     (jcp.t_pad == 0 && jcp.l_pad == 0)
-                            || (jcp.stride_w == 1 && jcp.stride_h == 1))
-            && IMPLICATION(mimo && !is_data_layout_nxc, jcp.ic % simd_w == 0);
-    if (!args_ok) return status::unimplemented;
+                            || (jcp.stride_w == 1 && jcp.stride_h == 1)),
+            VERBOSE_SHAPE_RESTRICTION);
+    VDISPATCH_CONV_IC(
+            IMPLICATION(mimo && !is_data_layout_nxc, jcp.ic % simd_w == 0),
+            VERBOSE_SHAPE_RESTRICTION);
 
     jcp.ic_tail = is_data_layout_nxc ? jcp.ic % simd_w : 0;
     jcp.oc_tail = is_data_layout_nxc
@@ -787,8 +797,9 @@ status_t jit_avx2_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
         r_pad_no_tail = nstl::max(0,
                 calculate_end_padding(jcp.l_pad, jcp.ow - jcp.ur_w_tail, jcp.iw,
                         jcp.stride_w, ext_kw));
-        if (jcp.ur_w < nstl::max(jcp.l_pad, r_pad_no_tail))
-            return status::unimplemented;
+        VDISPATCH_CONV_IC((jcp.ur_w >= nstl::max(jcp.l_pad, r_pad_no_tail)),
+                VERBOSE_UNSUPPORTED_PAD_FEATURE,
+                "width unroll exceeds padding size");
     }
     assert(jcp.nb_oc_blocking > 0);
     assert(jcp.ur_w * (jcp.nb_oc_blocking + 1) <= num_avail_regs);
@@ -1098,6 +1109,7 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &diff_src_d,
         const memory_desc_wrapper &weights_d,
         const memory_desc_wrapper &diff_dst_d) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(avx2)) return status::unimplemented;
 
     jcp.nthr = dnnl_get_max_threads();
@@ -1137,10 +1149,11 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     jcp.dilate_h = (ndims == 3) ? 0 : cd.dilates[ndims - 4];
     jcp.dilate_w = cd.dilates[ndims - 3];
 
-    if ((jcp.dilate_w != 0 && jcp.stride_w != 1)
+    const bool dilate_ok = !((jcp.dilate_w != 0 && jcp.stride_w != 1)
             || (jcp.dilate_d != 0 && jcp.stride_d != 1)
-            || (jcp.dilate_h != 0 && jcp.stride_h != 1))
-        return status::unimplemented;
+            || (jcp.dilate_h != 0 && jcp.stride_h != 1));
+    VDISPATCH_CONV_IC(dilate_ok, VERBOSE_UNSUPPORTED_FEATURE,
+            "unsupported shape with 'stride > 1' when 'dilate > 0'");
 
     const int simd_w = 8;
 
@@ -1169,8 +1182,9 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     bool ok_to_pad_channels = true && !is_data_layout_nxc && jcp.ngroups == 1;
 
     /* gemm-based convolution performs better in these cases */
-    if (jcp.ic < simd_w && jcp.kw > 3 && jcp.stride_w > 1)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(!(jcp.ic < simd_w && jcp.kw > 3 && jcp.stride_w > 1),
+            VERBOSE_IMPL_HEURISTIC_FAIL,
+            "failed heuristic with ic, kw and stride_w parameters");
 
     if (ok_to_pad_channels) {
         jcp.oc = rnd_up(jcp.oc, simd_w);
@@ -1203,7 +1217,7 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
             && jcp.oc <= diff_dst_d.padded_dims()[1]
             && jcp.dst_tag == required_dat_tag
             && jcp.src_tag == required_dat_tag && jcp.wei_tag == wei_tag;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
 
     const int ext_kw = calculate_extended_filter_size(jcp.kw, jcp.dilate_w);
     const int ext_kh = calculate_extended_filter_size(jcp.kh, jcp.dilate_h);
@@ -1219,7 +1233,8 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     bool kernel_outside_src = false || ext_kw <= jcp.l_pad
             || ext_kw <= jcp.r_pad || ext_kh <= jcp.t_pad || ext_kh <= jcp.b_pad
             || ext_kd <= jcp.f_pad || ext_kd <= jcp.back_pad;
-    if (kernel_outside_src) return status::unimplemented;
+    VDISPATCH_CONV_IC(!kernel_outside_src, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "weights and src size mismatch");
 
     int l_overflow = nstl::max(0, (ext_kw - 1 - jcp.l_pad) / jcp.stride_w);
 
@@ -1232,9 +1247,8 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
        per ur_w * nb_ic_blocking compute loops. Number of required registers
        is num_regs = ur_w * nb_ic_blocking + ur_w / stride_w <= max_regs.
        ur_w must be divisible by stride_w */
-    if (jcp.stride_w + 1 > max_regs) /* Minimal possible registers
-                                         distribution exceeds max_regs */
-        return status::unimplemented;
+    VDISPATCH_CONV_IC((jcp.stride_w + 1 <= max_regs), VERBOSE_BLOCKING_FAIL,
+            "blocking requirement exceeds max available registers");
 
     int best_nfmas = 0;
     for (int b = 1; b <= 4; b++) {
@@ -1255,8 +1269,8 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
             }
         }
     }
-    if (best_nfmas == 0) /* can't find appropriate blocking */
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(best_nfmas > 0, VERBOSE_BLOCKING_FAIL,
+            "cannot find appropriate blocking");
 
     jcp.ur_w_tail = jcp.iw % jcp.ur_w;
 
@@ -1270,7 +1284,8 @@ status_t jit_avx2_conv_bwd_data_kernel_f32::init_conf(jit_conv_conf_t &jcp,
             || ((jcp.iw > jcp.ur_w) && (jcp.ur_w % jcp.stride_w != 0))
             /* r_pad must not extend beyond ur_w_tail */
             || ((jcp.iw > jcp.ur_w) && (jcp.r_pad + jcp.ur_w_tail < 0));
-    if (tails_not_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(!tails_not_ok, VERBOSE_UNSUPPORTED_FEATURE,
+            "tail size unsupported");
 
     /* adjust the thread decomposition
      * to improve the perf for small problem size
@@ -1313,6 +1328,7 @@ status_t jit_avx2_conv_bwd_weights_kernel_f32::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
         const memory_desc_wrapper &diff_weights_d,
         const memory_desc_wrapper &diff_dst_d) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(avx2)) return status::unimplemented;
 
     const bool with_groups = diff_weights_d.ndims() == src_d.ndims() + 1;
@@ -1393,7 +1409,8 @@ status_t jit_avx2_conv_bwd_weights_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     const bool boundaries_ok = true && jcp.t_pad < max_h_pad
             && jcp.b_pad < max_h_pad && jcp.l_pad < max_w_pad
             && jcp.r_pad < max_w_pad && jcp.f_pad == 0 && jcp.back_pad == 0;
-    if (!boundaries_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(boundaries_ok, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "padding size unsupported (overflow)");
 
     bool ok_to_pad_channels = true && !is_data_layout_nxc && jcp.ngroups == 1;
 
@@ -1418,8 +1435,11 @@ status_t jit_avx2_conv_bwd_weights_kernel_f32::init_conf(jit_conv_conf_t &jcp,
                             && ((jcp.src_tag == dat_tag_nCx8c
                                         && jcp.dst_tag == dat_tag_nCx8c)
                                     || (jcp.src_tag == dat_tag_nxc
-                                            && jcp.dst_tag == dat_tag_nxc)))
-            && IMPLICATION(mimo && !is_data_layout_nxc, jcp.ic % simd_w == 0)
+                                            && jcp.dst_tag == dat_tag_nxc)));
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
+
+    bool params_ok
+            = IMPLICATION(mimo && !is_data_layout_nxc, jcp.ic % simd_w == 0)
             && IMPLICATION(!is_data_layout_nxc, jcp.oc % simd_w == 0)
             && jcp.kw < 14 && jcp.kh <= jcp.t_pad + jcp.ih /* [bwd_w:r1] */
             && jcp.kh <= jcp.ih /* [bwd_w:r2] */
@@ -1428,7 +1448,7 @@ status_t jit_avx2_conv_bwd_weights_kernel_f32::init_conf(jit_conv_conf_t &jcp,
             && jcp.dilate_d == 0 && jcp.dilate_h == 0 && jcp.dilate_w == 0
             && jcp.ic <= src_d.padded_dims()[1]
             && jcp.oc <= diff_dst_d.padded_dims()[1];
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(params_ok, VERBOSE_BAD_PARAM, "");
 
     jcp.ic_block = flat ? jcp.ic : simd_w;
     jcp.nb_ic = div_up(jcp.ic, jcp.ic_block);
