@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2022-2023 Intel Corporation
+ * Copyright 2022-2024 Intel Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,9 +25,8 @@
 
 #include <graph/utils/utils.hpp>
 
-#include "graph/interface/backend.hpp"
-
 #include "graph/backend/dnnl/common.hpp"
+#include "graph/backend/dnnl/dnnl_constant_tensor_cache.hpp"
 #include "graph/backend/dnnl/fusion_info.hpp"
 #include "graph/backend/dnnl/internal_attrs.hpp"
 #include "graph/backend/dnnl/op_executable.hpp"
@@ -329,7 +328,7 @@ matmul_executable_t::desc_t matmul_executable_t::create_desc(
                 pd_cache.at(op.get()));
         return {pd, true};
     }
-
+    const bool can_use_blocked_layout = mgr.get_use_blocked_layout();
     dnnl::primitive_attr prm_attr;
     if (op->has_attr(op_attr::fusion_info_key)
             && op->get_attr<int64_t>(op_attr::fusion_info_key) != -1) {
@@ -355,7 +354,7 @@ matmul_executable_t::desc_t matmul_executable_t::create_desc(
             = logical_tensor_wrapper_t(
                       op->get_input_value(0)->get_logical_tensor())
                       .is_constant()
-            && is_constant_cache_enabled();
+            && is_constant_cache_enabled(p_engine);
     const bool use_strided_src = !const_activation
             && ((src.get_ndims() == 4
                         && is_format(src, dnnl::memory::format_tag::acbd))
@@ -364,7 +363,9 @@ matmul_executable_t::desc_t matmul_executable_t::create_desc(
     // convert src memory desc to any when:
     // 1) not the situation mentioned above
     // 2) the given md is blocked and convert to queried layout is necessary
-    if (!use_strided_src || !is_plain(src)) { src = to_format_any(src); }
+    if (can_use_blocked_layout && (!use_strided_src || !is_plain(src))) {
+        src = to_format_any(src);
+    }
     auto wei = make_dnnl_memory_desc(
             op->get_input_value(1)->get_logical_tensor());
     // For non-constant weight, create primitive desc with strided layout when:
@@ -373,13 +374,14 @@ matmul_executable_t::desc_t matmul_executable_t::create_desc(
     bool const_weight = logical_tensor_wrapper_t(
                                 op->get_input_value(1)->get_logical_tensor())
                                 .is_constant()
-            && is_constant_cache_enabled();
-    const bool use_strided_wei = !const_weight
-            && (wei.get_ndims() == 4
-                    && (is_format(wei, dnnl::memory::format_tag::adbc)
-                            || is_format(wei, dnnl::memory::format_tag::abdc)
-                            || is_format(wei, dnnl::memory::format_tag::acbd)));
-    if (!use_strided_wei) { wei = to_format_any(wei); }
+            && is_constant_cache_enabled(p_engine);
+    const bool use_strided_wei = wei.get_ndims() == 4
+            && (is_format(wei, dnnl::memory::format_tag::adbc)
+                    || is_format(wei, dnnl::memory::format_tag::abdc)
+                    || is_format(wei, dnnl::memory::format_tag::acbd));
+    if (const_weight || (can_use_blocked_layout && !use_strided_wei)) {
+        wei = to_format_any(wei);
+    }
     auto dst = make_dnnl_memory_desc(
             op->get_output_value(0)->get_logical_tensor());
     const bool keep_dst_layout = op->has_attr(op_attr::keep_dst_layout)
@@ -388,7 +390,7 @@ matmul_executable_t::desc_t matmul_executable_t::create_desc(
             = ((src.get_ndims() == 2 || src.get_ndims() == 3)
                       && p_engine.get_kind() == dnnl::engine::kind::gpu)
             || keep_dst_layout;
-    if (!use_strided_dst) {
+    if (can_use_blocked_layout && !use_strided_dst) {
         dst = to_format_any(dst);
     } else if (dst.get_format_kind() == dnnl::memory::format_kind::any
             && !keep_dst_layout) {
@@ -752,9 +754,12 @@ layernorm_executable_t::desc_t layernorm_executable_t::create_desc(
 
     auto src = make_dnnl_memory_desc(
             op->get_input_value(0)->get_logical_tensor());
+    // onednn 3.6 spec: Implementations optimized for memory formats ab, abc,
+    // bac, abcd
+    src = to_ncx_format(src);
     auto dst = make_dnnl_memory_desc(
             op->get_output_value(0)->get_logical_tensor());
-
+    dst = to_format_any(dst);
     dnnl::layer_normalization_forward::primitive_desc pd(
             p_engine, pkind, src, dst, epsilon, flags, prm_attr);
 
@@ -1590,7 +1595,7 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     add_post_ops.append_eltwise(algorithm::eltwise_sqrt, 0.0f, 0.0f);
 
     primitive_attr add_attr;
-    add_attr.set_post_ops(add_post_ops);
+    add_attr.set_post_ops(std::move(add_post_ops));
     desc.add_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_add,
             variance, desc.epsilon_desc_, variance, add_attr);
 
@@ -1626,7 +1631,7 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     mul_post_ops.append_binary(algorithm::binary_div, desc.new_variance_desc_);
 
     primitive_attr mul_attr;
-    mul_attr.set_post_ops(mul_post_ops);
+    mul_attr.set_post_ops(std::move(mul_post_ops));
     desc.mul_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_mul,
             weights, desc.new_scale_desc_, weights, mul_attr);
 
@@ -1644,7 +1649,7 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     sub_post_ops.append_binary(algorithm::binary_add, shift);
 
     primitive_attr sub_attr;
-    sub_attr.set_post_ops(sub_post_ops);
+    sub_attr.set_post_ops(std::move(sub_post_ops));
     desc.sub_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_sub,
             valid_bias, mean, valid_bias, sub_attr);
 
@@ -1659,6 +1664,63 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     return desc;
 }
 
+groupnorm_executable_t::desc_t groupnorm_executable_t::create_desc(
+        std::shared_ptr<op_t> &op, const dnnl::engine &p_engine,
+        fusion_info_mgr_t &mgr, pd_cache_t &pd_cache) {
+
+    // first look up the cache
+    if (pd_cache.find(op.get()) != pd_cache.end()) {
+        auto pd = graph::utils::any_cast<
+                dnnl::group_normalization_forward::primitive_desc>(
+                pd_cache.at(op.get()));
+        return {pd, true};
+    }
+
+    dnnl::primitive_attr prm_attr;
+    if (op->has_attr(op_attr::fusion_info_key)
+            && op->get_attr<int64_t>(op_attr::fusion_info_key) != -1) {
+        int64_t key = op->get_attr<int64_t>(op_attr::fusion_info_key);
+        prm_attr = make_dnnl_primitive_attr(op, mgr.get_info(key));
+    }
+
+    prm_attr.set_scratchpad_mode(dnnl::scratchpad_mode::user);
+    float epsilon = 1e-5f;
+    if (op->has_attr(op_attr::epsilon))
+        epsilon = op->get_attr<float>(op_attr::epsilon);
+    bool keep_stats = true;
+    if (op->has_attr(op_attr::keep_stats))
+        keep_stats = op->get_attr<bool>(op_attr::keep_stats);
+    bool use_affine = true;
+    if (op->has_attr(op_attr::use_affine))
+        use_affine = op->get_attr<bool>(op_attr::use_affine);
+    int64_t group_num = 1;
+    if (op->has_attr(op_attr::groups)) {
+        group_num = op->get_attr<int64_t>(op_attr::groups);
+    } else {
+        assertm(false, "group_num is required.");
+    }
+    auto flags = dnnl::normalization_flags::none;
+    if (use_affine)
+        flags |= (dnnl::normalization_flags::use_scale
+                | dnnl::normalization_flags::use_shift);
+
+    prop_kind pkind = keep_stats ? prop_kind::forward_training
+                                 : prop_kind::forward_inference;
+
+    auto src = make_dnnl_memory_desc(
+            op->get_input_value(0)->get_logical_tensor());
+    auto dst = make_dnnl_memory_desc(
+            op->get_output_value(0)->get_logical_tensor());
+
+    dst = to_format_any(dst);
+
+    dnnl::group_normalization_forward::primitive_desc pd(
+            p_engine, pkind, src, dst, group_num, epsilon, flags, prm_attr);
+
+    pd_cache.insert({op.get(), pd});
+    return {pd, false};
+}
+
 static void get_arg_indices_for_post_ops(const op_t *op, fusion_info_mgr_t &mgr,
         arg_indices_t &indices, size_t &base_index) {
     const fusion_info_t &fusion_info
@@ -1678,6 +1740,10 @@ static void get_arg_indices_for_post_ops(const op_t *op, fusion_info_mgr_t &mgr,
         } else if (pops[i]->get_op()->get_kind() == op_kind::dnnl_convolution) {
             indices.insert({DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS,
                     indices_t {input, base_index++}});
+            if (pops[i]->get_op()->num_inputs() > 2) {
+                indices.insert({DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS,
+                        indices_t {input, base_index++}});
+            }
         } else {
         }
     }
@@ -1833,13 +1899,12 @@ static arg_indices_t get_arg_indices_for_siso_op(
             ? mgr.get_info(op->get_attr<int64_t>(op_attr::fusion_info_key))
             : fusion_info_t();
 
+    get_arg_indices_for_post_ops(op, mgr, arg_indices, index);
     if (fusion_info.with_runtime_scales(false, 0)) {
         arg_indices.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST,
                 indices_t {input, index++}});
     }
 
-    get_arg_indices_for_post_ops(op, mgr, arg_indices, index);
-
     // add output args
     arg_indices.insert({DNNL_ARG_TO, indices_t {output, 0}});
     arg_indices.insert({DNNL_ARG_SCRATCHPAD, indices_t {output, 1}});
@@ -2075,9 +2140,8 @@ arg_indices_t batchnorm_bwd_executable_t::get_arg_indices(
     return arg_indices;
 }
 
-arg_indices_t layernorm_executable_t::get_arg_indices(
+static arg_indices_t get_arg_indices_for_lnorm_and_gnorm(
         const op_t *op, fusion_info_mgr_t &mgr) {
-    UNUSED(mgr);
     arg_indices_t arg_indices;
 
     size_t in_index = 0;
@@ -2094,6 +2158,8 @@ arg_indices_t layernorm_executable_t::get_arg_indices(
             ? mgr.get_info(op->get_attr<int64_t>(op_attr::fusion_info_key))
             : fusion_info_t();
 
+    get_arg_indices_for_post_ops(op, mgr, arg_indices, in_index);
+
     if (fusion_info.with_runtime_scales(false, 0)) {
         arg_indices.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST,
                 indices_t {input, in_index++}});
@@ -2108,14 +2174,16 @@ arg_indices_t layernorm_executable_t::get_arg_indices(
                 {DNNL_ARG_VARIANCE, indices_t {output, out_index++}});
     }
 
-    if (op->num_outputs() > out_index) {
-        arg_indices.insert(
-                {DNNL_ARG_SCRATCHPAD, indices_t {output, out_index++}});
-    }
+    arg_indices.insert({DNNL_ARG_SCRATCHPAD, indices_t {output, out_index++}});
 
     return arg_indices;
 }
 
+arg_indices_t layernorm_executable_t::get_arg_indices(
+        const op_t *op, fusion_info_mgr_t &mgr) {
+    return get_arg_indices_for_lnorm_and_gnorm(op, mgr);
+}
+
 arg_indices_t layernorm_bwd_executable_t::get_arg_indices(
         const op_t *op, fusion_info_mgr_t &mgr) {
     arg_indices_t arg_indices;
@@ -2168,11 +2236,6 @@ arg_indices_t reorder_executable_t::get_arg_indices(
                 indices_t {input, index++}});
     }
 
-    if (fusion_info.with_runtime_scales(false, 0)) {
-        arg_indices.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST,
-                indices_t {input, index++}});
-    }
-
     if ((op->has_attr(op_attr::with_runtime_src_zps)
                 && op->get_attr<bool>(op_attr::with_runtime_src_zps))
             || fusion_info.with_runtime_zero_points(true, 0)) {
@@ -2182,6 +2245,11 @@ arg_indices_t reorder_executable_t::get_arg_indices(
 
     get_arg_indices_for_post_ops(op, mgr, arg_indices, index);
 
+    if (fusion_info.with_runtime_scales(false, 0)) {
+        arg_indices.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST,
+                indices_t {input, index++}});
+    }
+
     if ((op->has_attr(op_attr::with_runtime_dst_zps)
                 && op->get_attr<bool>(op_attr::with_runtime_dst_zps))
             || fusion_info.with_runtime_zero_points(false, 0)) {
@@ -2241,6 +2309,11 @@ arg_indices_t eltwise_bwd_executable_t::get_arg_indices(
     return arg_indices;
 }
 
+arg_indices_t groupnorm_executable_t::get_arg_indices(
+        const op_t *op, fusion_info_mgr_t &mgr) {
+    return get_arg_indices_for_lnorm_and_gnorm(op, mgr);
+}
+
 } // namespace dnnl_impl
 } // namespace graph
 } // namespace impl
