@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2023 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,6 +27,31 @@ namespace graph {
 namespace dnnl_impl {
 namespace pattern {
 
+namespace {
+
+bool check_s8_weight(op_t *op) {
+    const op_kind_t kind = op->get_kind();
+    if (kind != graph::op_kind::Dequantize) return true;
+    return (!is_int8_quantization(op)
+            || check_input_dtype<graph::data_type::s8>(op));
+}
+
+bool check_f8_conv_with_no_scale_and_zp(op_t *op) {
+    const op_kind_t kind = op->get_kind();
+    if (kind != graph::op_kind::Convolution) return true;
+
+    for (const size_t index : {0, 1}) {
+        auto &parent_op = op->get_input_value(index)->get_producer();
+        if (parent_op.get_kind() != graph::op_kind::Dequantize) return true;
+        if (is_f8_quantization(&parent_op)
+                && !check_quant_with_no_effect(&parent_op))
+            return false;
+    }
+    return true;
+}
+
+} // namespace
+
 namespace pm = graph::utils::pm;
 using in_edges_t = pm::in_edges_t;
 using pb_graph_t = pm::pb_graph_t;
@@ -41,28 +66,71 @@ DNNL_BACKEND_REGISTER_PATTERN_DEF_BEGIN(conv_post_ops)
               \   /
               conv
                 |
-         depthwise_conv
+             [bias]*
                 |
+    [unary/binary]*[0,MAX_REPETITION)
+                |
+          depthwise_conv
+                |
+             [bias]*
+                |
+    [unary/binary]*[0,MAX_REPETITION)
+                |
+
+  (TODO:Jiexin)limitation: For now, this pattern use decision function 
+  check_conv_weight_size to avoid mismatching, but this implicitly requires user 
+  to provide conv weight size when creating logical tensors if they want the 
+  computation graph to benefit from this pattern.
 */
-DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_depthwise_cpu)
-        .set_priority(10.2f)
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(
+        dnnl, fp_conv_postops_depthwise_postops_cpu)
+        .set_priority(10.3f)
         .set_engine_kind(engine_kind::cpu)
         .set_kind(partition_kind_t::convolution_post_ops)
         .set_attr<FCreatePattern>("FCreatePattern",
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
-                    pm::pb_op_t *conv
+                    pm::pb_op_t *pconv
                             = pgraph->append_op(graph::op_kind::Convolution);
-                    conv->append_decision_function(check_input_num<2>);
+                    pconv->append_decision_function(check_conv_weight_size<1>);
+                    // Optional bias_add
+                    auto popt_bias1 = optional_bias_add(pgraph, pconv, false);
+
+                    auto alt_graph1 = std::make_shared<pb_graph_t>();
+                    auto palt1 = alt_graph1->append_alternation(
+                            get_unary_binary_ops());
+                    palt1->allow_internal_inputs();
+                    alt_graph1->create_input_port(0, palt1, 0);
+                    alt_graph1->create_output_port(0, palt1, 0);
+
+                    auto prep1 = pgraph->append_repetition(alt_graph1, {0, 0},
+                            0, MAX_REPETITION,
+                            in_edges_t {in_edge(0, popt_bias1, 0)});
 
                     pm::pb_op_t *depthwise
                             = pgraph->append_op(graph::op_kind::Convolution,
-                                    in_edges_t {in_edge(0, conv, 0)});
-                    depthwise->append_decision_function(check_input_num<2>);
+                                    in_edges_t {in_edge(0, prep1, 0)});
+                    depthwise->append_decision_function(
+                            check_conv_weight_size<3>);
+                    // Optional bias_add
+                    auto popt_bias2
+                            = optional_bias_add(pgraph, depthwise, false);
+
+                    auto alt_graph2 = std::make_shared<pb_graph_t>();
+                    auto palt2 = alt_graph2->append_alternation(
+                            get_unary_binary_ops());
+                    palt2->allow_internal_inputs();
+                    alt_graph2->create_input_port(0, palt2, 0);
+                    alt_graph2->create_output_port(0, palt2, 0);
+
+                    pgraph->append_repetition(alt_graph2, {0, 0}, 0,
+                            MAX_REPETITION,
+                            in_edges_t {in_edge(0, popt_bias2, 0)});
                 })
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<float_conv_fwd>();
         });
-
+#endif
 /*
                     [quant_weight]*
         |                  |
@@ -75,7 +143,7 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_depthwise_cpu)
                 |  /
                add
                 |
-        [unary/binary]*[0,4]
+[unary/binary]*[0,MAX_REPETITION)
                 |
             quant_out
                 |
@@ -85,7 +153,8 @@ Conv: Currently DNNL Backend doesn't support below
 features on GPU:
 1. Post-sum with zero points
 */
-DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_cpu)
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8x8x8_conv_add_post_ops_cpu)
         .set_priority(10.6f)
         .set_engine_kind(engine_kind::cpu)
         .set_kind(partition_kind_t::quantized_convolution_post_ops)
@@ -98,6 +167,7 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_cpu)
                     auto popt_graph = std::make_shared<pb_graph_t>();
                     pm::pb_op_t *pquant
                             = popt_graph->append_op(graph::op_kind::Quantize);
+                    pquant->append_decision_function(is_int8_quantization);
                     popt_graph->create_input_port(0, pquant, 0);
                     popt_graph->create_output_port(0, pquant, 0);
                     auto popt = pgraph->append_optional(popt_graph);
@@ -106,13 +176,14 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_cpu)
                             = pgraph->append_op(graph::op_kind::Dequantize,
                                     in_edges_t {in_edge(0, popt, 0)});
                     // Currently oneDNN convolution primitive only supports s8 weight
-                    dequant_weight->append_decision_function(
-                            check_input_dtype<graph::data_type::s8>);
+                    dequant_weight->append_decision_function(check_s8_weight);
 
                     pm::pb_op_t *pconv
                             = pgraph->append_op(graph::op_kind::Convolution,
                                     in_edges_t {in_edge(0, dequant_data, 0),
                                             in_edge(1, dequant_weight, 0)});
+                    pconv->append_decision_function(
+                            check_f8_conv_with_no_scale_and_zp);
 
                     // Optional bias_add
                     auto popt_bias = optional_bias_add(pgraph, pconv, false);
@@ -127,8 +198,10 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_cpu)
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<quantized_conv>();
         });
+#endif
 
-DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_gpu)
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE
+DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8x8x8_conv_add_post_ops_gpu)
         .set_priority(10.6f)
         .set_engine_kind(engine_kind::gpu)
         .set_kind(partition_kind_t::quantized_convolution_post_ops)
@@ -149,13 +222,14 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_gpu)
                             = pgraph->append_op(graph::op_kind::Dequantize,
                                     in_edges_t {in_edge(0, popt, 0)});
                     // Currently oneDNN convolution primitive only supports s8 weight
-                    dequant_weight->append_decision_function(
-                            check_input_dtype<graph::data_type::s8>);
+                    dequant_weight->append_decision_function(check_s8_weight);
 
                     pm::pb_op_t *pconv
                             = pgraph->append_op(graph::op_kind::Convolution,
                                     in_edges_t {in_edge(0, dequant_data, 0),
                                             in_edge(1, dequant_weight, 0)});
+                    pconv->append_decision_function(
+                            check_f8_conv_with_no_scale_and_zp);
 
                     // Optional bias_add
                     auto popt_bias = optional_bias_add(pgraph, pconv, false);
@@ -171,6 +245,7 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_gpu)
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<quantized_conv>();
         });
+#endif
 
 /*
                     [quant_weight]*
@@ -181,12 +256,12 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x8_conv_add_post_ops_gpu)
                 |
               [bias]*
                 |
-        [unary/binary]*[0,4]
+[unary/binary]*[0,MAX_REPETITION)
                 |
             [quant_out]*
                 |
 */
-DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_conv_post_ops)
+DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8x8x_conv_post_ops)
         .set_priority(10.5f)
         .set_kind(partition_kind_t::quantized_convolution_post_ops)
         .set_attr<FCreatePattern>("FCreatePattern",
@@ -206,13 +281,14 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_conv_post_ops)
                             = pgraph->append_op(graph::op_kind::Dequantize,
                                     in_edges_t {in_edge(0, popt, 0)});
                     // Currently oneDNN convolution primitive only supports s8 weight
-                    dequant_weight->append_decision_function(
-                            check_input_dtype<graph::data_type::s8>);
+                    dequant_weight->append_decision_function(check_s8_weight);
 
                     pm::pb_op_t *pconv
                             = pgraph->append_op(graph::op_kind::Convolution,
                                     in_edges_t {in_edge(0, dequant_data, 0),
                                             in_edge(1, dequant_weight, 0)});
+                    pconv->append_decision_function(
+                            check_f8_conv_with_no_scale_and_zp);
 
                     // Optional bias_add
                     auto popt_bias = optional_bias_add(pgraph, pconv, false);
@@ -256,7 +332,7 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_conv_post_ops)
                 |
               [bias]*
                 |           
-        [unary/binary]*[0,4]
+[unary/binary]*[0,MAX_REPETITION)
                 |
             [quant_out]*
                 | 
@@ -271,6 +347,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_conv_reshape_post_ops)
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
                     pm::pb_op_t *dequant_data
                             = pgraph->append_op(graph::op_kind::Dequantize);
+                    dequant_data->append_decision_function(
+                            is_int8_quantization);
 
                     pm::pb_op_t *weight_reshape1
                             = pgraph->append_op(graph::op_kind::StaticReshape);
@@ -337,16 +415,19 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_conv_reshape_post_ops)
                 |          /
                 |  typecast_other
                 |  /
-               Add   [typecast_binary]*
-                |        /
-        [unary/binary]*[0,4]
+               Add
                 |
-    [typecast_out -> quant_out]*
+[unary/binary]*[0,MAX_REPETITION)
+                |
+            typecast_out
+                |
+            quant_out
 
 Conv: Currently DNNL Backend doesn't support below
 features on GPU:
 1. Post-sum with zero points
 */
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
 DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_cpu)
         .set_priority(10.6f)
         .set_engine_kind(engine_kind::cpu)
@@ -355,6 +436,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_cpu)
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
                     pm::pb_op_t *dequant_data
                             = pgraph->append_op(graph::op_kind::Dequantize);
+                    dequant_data->append_decision_function(
+                            is_int8_quantization);
                     pm::pb_op_t *typecast_data
                             = pgraph->append_op(graph::op_kind::TypeCast,
                                     in_edges_t {in_edge(0, dequant_data, 0)});
@@ -398,6 +481,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_cpu)
                     // post add with dequant->typecast
                     pm::pb_op_t *pdequant_add
                             = pgraph->append_op(graph::op_kind::Dequantize);
+                    pdequant_add->append_decision_function(
+                            is_int8_quantization);
                     pm::pb_op_t *typecast_add
                             = pgraph->append_op(graph::op_kind::TypeCast,
                                     in_edges_t {in_edge(0, pdequant_add, 0)});
@@ -406,57 +491,35 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_cpu)
                                     in_edge(1, typecast_add, 0)});
 
                     //post ops
-                    auto postop_with_tc_graph = std::make_shared<pb_graph_t>();
+                    auto postop_graph = std::make_shared<pb_graph_t>();
+                    pm::pb_op_t *pop = postop_graph->append_alternation(
+                            get_unary_binary_ops());
+                    postop_graph->create_input_port(0, pop, 0);
+                    postop_graph->create_input_port(1, pop, 1);
+                    postop_graph->create_output_port(0, pop, 0);
 
-                    auto popt_typecast_binary_graph
-                            = std::make_shared<pb_graph_t>();
-                    pm::pb_op_t *typecast_binary
-                            = popt_typecast_binary_graph->append_op(
-                                    graph::op_kind::TypeCast);
-                    popt_typecast_binary_graph->create_input_port(
-                            0, typecast_binary, 0);
-                    popt_typecast_binary_graph->create_output_port(
-                            0, typecast_binary, 0);
-                    auto popt_typecast_binary
-                            = postop_with_tc_graph->append_optional(
-                                    popt_typecast_binary_graph);
-
-                    pm::pb_op_t *pop_with_tc
-                            = postop_with_tc_graph->append_alternation(
-                                    get_unary_binary_ops(),
-                                    in_edges_t {in_edge(
-                                            1, popt_typecast_binary, 0)});
-                    postop_with_tc_graph->create_input_port(0, pop_with_tc, 0);
-                    postop_with_tc_graph->create_input_port(
-                            1, popt_typecast_binary, 1);
-                    postop_with_tc_graph->create_output_port(0, pop_with_tc, 0);
-
-                    auto prep = pgraph->append_repetition(postop_with_tc_graph,
-                            {0, 0}, 0, MAX_REPETITION,
+                    auto prep = pgraph->append_repetition(postop_graph, {0, 0},
+                            0, MAX_REPETITION,
                             in_edges_t {in_edge(0, padd, 0)});
 
-                    // Optional typecast_out + quant_out
-                    auto popt_tcout_qout_graph = std::make_shared<pb_graph_t>();
-                    pm::pb_op_t *ptc_out = popt_tcout_qout_graph->append_op(
-                            graph::op_kind::TypeCast);
+                    pm::pb_op_t *ptc_out
+                            = pgraph->append_op(graph::op_kind::TypeCast,
+                                    in_edges_t {in_edge(0, prep, 0)});
                     // TODO: remove bf16 datatype check when all float point
                     // datatypes are enabled
                     ptc_out->append_decision_function(
                             check_input_dtype<graph::data_type::bf16>);
                     ptc_out->append_decision_function(
                             check_output_dtype<graph::data_type::f32>);
-                    pm::pb_op_t *pquant_out = popt_tcout_qout_graph->append_op(
-                            graph::op_kind::Quantize,
+                    pgraph->append_op(graph::op_kind::Quantize,
                             in_edges_t {in_edge(0, ptc_out, 0)});
-                    popt_tcout_qout_graph->create_input_port(0, ptc_out, 0);
-                    popt_tcout_qout_graph->create_output_port(0, pquant_out, 0);
-                    pgraph->append_optional(popt_tcout_qout_graph,
-                            in_edges_t {in_edge(0, prep, 0)});
                 })
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<quantized_conv>();
         });
+#endif
 
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE
 DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_gpu)
         .set_priority(10.6f)
         .set_engine_kind(engine_kind::gpu)
@@ -465,6 +528,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_gpu)
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
                     pm::pb_op_t *dequant_data
                             = pgraph->append_op(graph::op_kind::Dequantize);
+                    dequant_data->append_decision_function(
+                            is_int8_quantization);
                     pm::pb_op_t *typecast_data
                             = pgraph->append_op(graph::op_kind::TypeCast,
                                     in_edges_t {in_edge(0, dequant_data, 0)});
@@ -509,6 +574,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_gpu)
                     pm::pb_op_t *pdequant_add
                             = pgraph->append_op(graph::op_kind::Dequantize);
                     pdequant_add->append_decision_function(check_zps_values<0>);
+                    pdequant_add->append_decision_function(
+                            is_int8_quantization);
                     pm::pb_op_t *typecast_add
                             = pgraph->append_op(graph::op_kind::TypeCast,
                                     in_edges_t {in_edge(0, pdequant_add, 0)});
@@ -517,58 +584,38 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_gpu)
                                     in_edge(1, typecast_add, 0)});
 
                     //post ops
-                    auto postop_with_tc_graph = std::make_shared<pb_graph_t>();
+                    auto postop_graph = std::make_shared<pb_graph_t>();
+                    pm::pb_op_t *pop = postop_graph->append_alternation(
+                            get_unary_binary_ops());
+                    postop_graph->create_input_port(0, pop, 0);
+                    postop_graph->create_input_port(1, pop, 1);
+                    postop_graph->create_output_port(0, pop, 0);
 
-                    auto popt_typecast_binary_graph
-                            = std::make_shared<pb_graph_t>();
-                    pm::pb_op_t *typecast_binary
-                            = popt_typecast_binary_graph->append_op(
-                                    graph::op_kind::TypeCast);
-                    popt_typecast_binary_graph->create_input_port(
-                            0, typecast_binary, 0);
-                    popt_typecast_binary_graph->create_output_port(
-                            0, typecast_binary, 0);
-                    auto popt_typecast_binary
-                            = postop_with_tc_graph->append_optional(
-                                    popt_typecast_binary_graph);
-
-                    pm::pb_op_t *pop_with_tc
-                            = postop_with_tc_graph->append_alternation(
-                                    get_unary_binary_ops(),
-                                    in_edges_t {in_edge(
-                                            1, popt_typecast_binary, 0)});
-                    postop_with_tc_graph->create_input_port(0, pop_with_tc, 0);
-                    postop_with_tc_graph->create_input_port(
-                            1, popt_typecast_binary, 1);
-                    postop_with_tc_graph->create_output_port(0, pop_with_tc, 0);
-
-                    auto prep = pgraph->append_repetition(postop_with_tc_graph,
-                            {0, 0}, 0, MAX_REPETITION,
+                    auto prep = pgraph->append_repetition(postop_graph, {0, 0},
+                            0, MAX_REPETITION,
                             in_edges_t {in_edge(0, padd, 0)});
 
-                    // Optional typecast_out + quant_out
-                    auto popt_tcout_qout_graph = std::make_shared<pb_graph_t>();
-                    pm::pb_op_t *ptc_out = popt_tcout_qout_graph->append_op(
-                            graph::op_kind::TypeCast);
+                    pm::pb_op_t *ptc_out
+                            = pgraph->append_op(graph::op_kind::TypeCast,
+                                    in_edges_t {in_edge(0, prep, 0)});
                     // TODO: remove bf16 datatype check when all float point
                     // datatypes are enabled
                     ptc_out->append_decision_function(
                             check_input_dtype<graph::data_type::bf16>);
                     ptc_out->append_decision_function(
                             check_output_dtype<graph::data_type::f32>);
-                    pm::pb_op_t *pquant_out = popt_tcout_qout_graph->append_op(
-                            graph::op_kind::Quantize,
+                    pgraph->append_op(graph::op_kind::Quantize,
                             in_edges_t {in_edge(0, ptc_out, 0)});
-                    popt_tcout_qout_graph->create_input_port(0, ptc_out, 0);
-                    popt_tcout_qout_graph->create_output_port(0, pquant_out, 0);
-                    pgraph->append_optional(popt_tcout_qout_graph,
-                            in_edges_t {in_edge(0, prep, 0)});
                 })
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<quantized_conv>();
         });
-
+#endif
 /*
+TODO(Jiexin): Currently, we found conv+relu+multiply/divide/subtract lacks 
+optimization in oneDNN post-ops fusion, so we don't plan to extend the post-op 
+chain of this pattern, we will extend it when corresponding optimization is 
+enabled.
                     [quant_weight]*
         |                  |
    dequant_data     dequant_weight
@@ -578,9 +625,9 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_add_post_ops_gpu)
                conv
                 | [typecast_bias]*
                 |   /
-              [bias]*  [typecast_binary]*
-                |           /
-        [unary/binary]*[0,4]
+              [bias]*
+                |
+            [unary/binary]*[0,MAX_REPETITION)
                 |
     [typecast_out -> quant_out]*
                 |
@@ -592,11 +639,14 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_post_ops)
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
                     pm::pb_op_t *dequant_data
                             = pgraph->append_op(graph::op_kind::Dequantize);
+                    dequant_data->append_decision_function(
+                            is_int8_quantization);
                     // Optional quant_weight
                     auto popt_graph = std::make_shared<pb_graph_t>();
                     pm::pb_op_t *pquant
                             = popt_graph->append_op(graph::op_kind::Quantize);
                     pquant->append_decision_function(check_if_constant_weight);
+                    pquant->append_decision_function(is_int8_quantization);
                     popt_graph->create_input_port(0, pquant, 0);
                     popt_graph->create_output_port(0, pquant, 0);
                     auto popt = pgraph->append_optional(popt_graph);
@@ -627,33 +677,16 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_post_ops)
                     auto popt_bias
                             = optional_bias_add(pgraph, convolution, true);
                     // post ops
-                    auto postop_with_tc_graph = std::make_shared<pb_graph_t>();
+                    auto postop_graph = std::make_shared<pb_graph_t>();
+                    pm::pb_op_t *pop = postop_graph->append_alternation(
+                            get_unary_binary_ops());
+                    pop->allow_internal_inputs();
+                    postop_graph->create_input_port(0, pop, 0);
+                    postop_graph->create_input_port(1, pop, 1);
+                    postop_graph->create_output_port(0, pop, 0);
 
-                    auto popt_typecast_binary_graph
-                            = std::make_shared<pb_graph_t>();
-                    pm::pb_op_t *typecast_binary
-                            = popt_typecast_binary_graph->append_op(
-                                    graph::op_kind::TypeCast);
-                    popt_typecast_binary_graph->create_input_port(
-                            0, typecast_binary, 0);
-                    popt_typecast_binary_graph->create_output_port(
-                            0, typecast_binary, 0);
-                    auto popt_typecast_binary
-                            = postop_with_tc_graph->append_optional(
-                                    popt_typecast_binary_graph);
-
-                    pm::pb_op_t *pop_with_tc
-                            = postop_with_tc_graph->append_alternation(
-                                    get_unary_binary_ops(),
-                                    in_edges_t {in_edge(
-                                            1, popt_typecast_binary, 0)});
-                    postop_with_tc_graph->create_input_port(0, pop_with_tc, 0);
-                    postop_with_tc_graph->create_input_port(
-                            1, popt_typecast_binary, 1);
-                    postop_with_tc_graph->create_output_port(0, pop_with_tc, 0);
-
-                    auto prep = pgraph->append_repetition(postop_with_tc_graph,
-                            {0, 0}, 0, MAX_REPETITION,
+                    auto prep = pgraph->append_repetition(postop_graph, {0, 0},
+                            0, MAX_REPETITION,
                             in_edges_t {in_edge(0, popt_bias, 0)});
 
                     // Optional typecast_out + quant_out
@@ -686,7 +719,7 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, x8s8x_tc_conv_post_ops)
                 |
               [BN]*
                 |
-        [unary/binary]*[0,4]
+[unary/binary]*[0,MAX_REPETITION)
                 |
            [TypeCast]*
                 |
@@ -745,7 +778,8 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_post_ops)
       conv_bwd_weight  biasadd_bwd
                 |          |
 */
-DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_bwd_weights_bwd_bias)
+#if BUILD_TRAINING
+DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_bwd_weights_bias)
         .set_enable(false)
         .set_kind(partition_kind_t::convolution_backward_post_ops)
         .set_priority(9.7f)
@@ -753,15 +787,19 @@ DNNL_BACKEND_REGISTER_PATTERN_MATCHER_PASS(dnnl, fp_conv_bwd_weights_bwd_bias)
                 [](const std::shared_ptr<pb_graph_t> &pgraph) -> void {
                     pm::pb_op_t *wildcard
                             = pgraph->append_op(graph::op_kind::Wildcard);
-                    pgraph->append_op(
-                            graph::op_kind::ConvolutionBackwardWeights,
-                            in_edges_t {in_edge(1, wildcard, 0)});
+                    graph::utils::pm::pb_op_t *p_conv_backward_weights
+                            = pgraph->append_op(
+                                    graph::op_kind::ConvolutionBackwardWeights,
+                                    in_edges_t {in_edge(1, wildcard, 0)});
+                    p_conv_backward_weights->append_decision_function(
+                            check_input_num<2>);
                     pgraph->append_op(graph::op_kind::BiasAddBackward,
                             in_edges_t {in_edge(0, wildcard, 0)});
                 })
         .set_attr<FCreateKernel>("FCreateKernel", []() -> kernel_ptr {
             return std::make_shared<conv_bwd_weights_t>();
         });
+#endif
 
 DNNL_BACKEND_REGISTER_PATTERN_DEF_END
 
