@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2023 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -67,17 +67,28 @@ const char *state2str(res_state_t state) {
     return "STATE_UNDEF";
 }
 
-const char *skip_reason2str(skip_reason_t skip_reason) {
+namespace skip_reason {
+std::string case_not_supported("Case not supported");
+std::string data_type_not_supported("Data type not supported");
+std::string invalid_case("Invalid case");
+std::string not_enough_ram("Not enough RAM");
+std::string skip_impl_hit("Skip-impl option hit");
+std::string skip_start("Skip-start option hit");
+} // namespace skip_reason
+
+dir_t str2dir(const char *str) {
 #define CASE(x) \
-    if (skip_reason == (x)) return STRINGIFY(x)
-    CASE(CASE_NOT_SUPPORTED);
-    CASE(DATA_TYPE_NOT_SUPPORTED);
-    CASE(INVALID_CASE);
-    CASE(NOT_ENOUGH_RAM);
-    CASE(SKIP_IMPL_HIT);
-    CASE(SKIP_START);
+    if (!strcasecmp(STRINGIFY(x), str)) return x
+    CASE(FWD_D);
+    CASE(FWD_I);
+    CASE(FWD_B);
+    CASE(BWD_D);
+    CASE(BWD_W);
+    CASE(BWD_WB);
+    CASE(BWD_DW);
 #undef CASE
-    return "SKIP_UNKNOWN";
+    assert(!"unknown dir");
+    return DIR_UNDEF;
 }
 
 void parse_result(res_t &res, const char *pstr) {
@@ -94,15 +105,22 @@ void parse_result(res_t &res, const char *pstr) {
                 BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.passed++;
             break;
-        case FAILED:
+        case FAILED: {
             bs.failed++;
-            BENCHDNN_PRINT(0, "%d:%s (errors:%lu total:%lu) __REPRO: %s\n",
-                    bs.tests, state, (unsigned long)res.errors,
-                    (unsigned long)res.total, pstr);
-            break;
+            std::string error_stat;
+            if (res.errors > 0) {
+                error_stat = " (errors:" + std::to_string(res.errors)
+                        + " total:" + std::to_string(res.total) + ")";
+            }
+
+            std::string reason;
+            if (!res.reason.empty()) { reason = " (" + res.reason + ")"; }
+            BENCHDNN_PRINT(0, "%d:%s%s%s __REPRO: %s\n", bs.tests, state,
+                    reason.c_str(), error_stat.c_str(), pstr);
+        } break;
         case SKIPPED:
             BENCHDNN_PRINT(0, "%d:%s (%s) __REPRO: %s\n", bs.tests, state,
-                    skip_reason2str(res.reason), pstr);
+                    res.reason.c_str(), pstr);
             bs.skipped++;
             break;
         case UNIMPLEMENTED:
@@ -322,13 +340,13 @@ bool maybe_skip(const std::string &impl_str) {
 bool skip_start(res_t *res, int idx) {
     if (idx < test_start) {
         res->state = SKIPPED;
-        res->reason = SKIP_START;
+        res->reason = skip_reason::skip_start;
         return true;
     }
     return false;
 }
 
-#if defined(_WIN32) && !defined(__GNUC__)
+#if defined(_WIN32)
 #include <windows.h>
 #define PATH_MAX MAX_PATH
 static char *dirname(char *path) {
@@ -639,3 +657,63 @@ void print_dhw(bool &print_d, bool &print_h, bool &print_w, int ndims,
     print_w = ndims == 3 || (ndims == 5 && (!cubic_shape || canonical))
             || (ndims == 4 && (!square_shape || canonical));
 }
+
+// Copied from utils::getenv.
+// An underlined unified implementation for getting env var value.
+int getenv(const char *name, char *buffer, int buffer_size) {
+    if (name == nullptr || buffer_size < 0
+            || (buffer == nullptr && buffer_size > 0))
+        return INT_MIN;
+
+    int result = 0;
+    int term_zero_idx = 0;
+    size_t value_length = 0;
+
+#ifdef _WIN32
+    value_length = GetEnvironmentVariable(name, buffer, buffer_size);
+#else
+    const char *value = ::getenv(name);
+    value_length = value == nullptr ? 0 : strlen(value);
+#endif
+
+    if (value_length > INT_MAX)
+        result = INT_MIN;
+    else {
+        int int_value_length = (int)value_length;
+        if (int_value_length >= buffer_size) {
+            result = -int_value_length;
+        } else {
+            term_zero_idx = int_value_length;
+            result = int_value_length;
+#ifndef _WIN32
+            if (value) strncpy(buffer, value, buffer_size - 1);
+#endif
+        }
+    }
+
+    if (buffer != nullptr) buffer[term_zero_idx] = '\0';
+    return result;
+}
+
+// Copied from utils::getenv_int_user.
+// Collects an integer value from an env var.
+int benchdnn_getenv_int(const char *name, int default_value) {
+    int value = default_value;
+    // # of digits in the longest 32-bit signed int + sign + terminating null
+    const int len = 12;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = atoi(value_str); }
+    return value;
+}
+
+// Copied from utils::getenv_string_user.
+// Collects a string lower case value from an env var.
+std::string benchdnn_getenv_string(const char *name) {
+    // Random number to fit possible string input.
+    std::string value;
+    const int len = 128;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = value_str; }
+    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+    return value;
+}
