@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2023 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -33,6 +33,8 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
             = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
     const dnn_mem_t &src_zps
             = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_WEIGHTS);
     const dnn_mem_t &dst_zps
             = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
@@ -44,12 +46,16 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
     float src_scale = has_src_scale ? src_scales.get_elem(0) : 1.f;
     float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
     const int wei_scale_mask = prb->attr.scales.get_mask(
-            DNNL_ARG_WEIGHTS, dnnl_convolution, prb->has_groups);
+            DNNL_ARG_WEIGHTS, dnnl_convolution, wei_m.ndims(), prb->has_groups);
 
     const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_zp
+            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
     const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
     const int src_zp_mask = attr_t::get_default_mask(
             prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int wei_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).policy);
     const int dst_zp_mask = attr_t::get_default_mask(
             prb->attr.zero_points.get(DNNL_ARG_DST).policy);
 
@@ -65,6 +71,8 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
     const int64_t DH = prb->dh + 1;
     const int64_t DW = prb->dw + 1;
 
+    int wei_zp = (has_wei_zp && (wei_zp_mask == 0)) ? wei_zps.get_elem(0) : 0;
+
     auto ker = [&](float &d, int64_t g, int64_t mb, int64_t oc, int64_t od,
                        int64_t oh, int64_t ow) {
         const float *__restrict src_loc
@@ -90,7 +98,7 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
                                                 : 0;
                         const float s = src_loc[src_off];
                         const float w = wei_loc[wei_off];
-                        const float d_tmp = (s - src_zp) * w;
+                        const float d_tmp = (s - src_zp) * (w - wei_zp);
                         d += d_tmp;
                     }
                 }
@@ -147,6 +155,8 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
             = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
     const dnn_mem_t &src_zps
             = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_WEIGHTS);
     const dnn_mem_t &dst_zps
             = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
@@ -158,12 +168,16 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
     float src_scale = has_src_scale ? src_scales.get_elem(0) : 1.f;
     float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
     const int wei_scale_mask = prb->attr.scales.get_mask(
-            DNNL_ARG_WEIGHTS, dnnl_convolution, prb->has_groups);
+            DNNL_ARG_WEIGHTS, dnnl_convolution, wei_m.ndims(), prb->has_groups);
 
     const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_zp
+            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
     const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
     const int src_zp_mask = attr_t::get_default_mask(
             prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int wei_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).policy);
     const int dst_zp_mask = attr_t::get_default_mask(
             prb->attr.zero_points.get(DNNL_ARG_DST).policy);
 
@@ -179,6 +193,8 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
     const int64_t DH = prb->dh + 1;
     const int64_t DW = prb->dw + 1;
 
+    int wei_zp = (has_wei_zp && (wei_zp_mask == 0)) ? wei_zps.get_elem(0) : 0;
+
     enum { precompute_size = 16 };
     const bool fast = MAX3(KD, KH, KW) <= precompute_size;
 
@@ -231,7 +247,7 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
             if (has_wei_scale)
                 wei_scale = wei_scales.get_elem(
                         wei_scale_mask > 0 ? g * ICG + ic : 0);
-            float wei_val = wei_loc[wei_off] * wei_scale;
+            float wei_val = (wei_loc[wei_off] - wei_zp) * wei_scale;
             ds += diff_dst_val * wei_val;
         }
     };
@@ -271,7 +287,7 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
                         if (has_wei_scale)
                             wei_scale = wei_scales.get_elem(
                                     wei_scale_mask > 0 ? g * ICG + ic : 0);
-                        float wei_val = wei_loc[wei_off] * wei_scale;
+                        float wei_val = (wei_loc[wei_off] - wei_zp) * wei_scale;
                         ds += diff_dst_val * wei_val;
                     }
                 }
