@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2023 Intel Corporation
+* Copyright 2022-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,17 +18,44 @@
 #include <set>
 #include <vector>
 
-#include "oneapi/dnnl/dnnl_graph.h"
-
 #include "cpu/platform.hpp"
-#ifdef DNNL_WITH_SYCL
-#include "dnnl_sycl.hpp"
-#endif
+
+#include "allocator.hpp"
 #include "utils.hpp"
 #include "utils/timer.hpp"
 
 namespace graph {
 
+bdnn_state_t convert_state(const dnnl_status_t &s) {
+    switch (s) {
+        case dnnl_status_t::dnnl_success:
+            return bdnn_state_t {res_state_t::PASSED};
+        case dnnl_status_t::dnnl_out_of_memory:
+            return bdnn_state_t {
+                    res_state_t::SKIPPED, skip_reason::not_enough_ram};
+        case dnnl_status_t::dnnl_invalid_arguments:
+            return bdnn_state_t {res_state_t::INVALID_ARGUMENTS};
+        case dnnl_status_t::dnnl_unimplemented:
+            return bdnn_state_t {res_state_t::UNIMPLEMENTED};
+        case dnnl_status_t::dnnl_last_impl_reached:
+            return bdnn_state_t {
+                    res_state_t::SKIPPED, skip_reason::skip_impl_hit};
+        case dnnl_status_t::dnnl_runtime_error:
+            return bdnn_state_t {res_state_t::FAILED};
+        case dnnl_status_t::dnnl_not_required:
+            return bdnn_state_t {res_state_t::INVALID_ARGUMENTS};
+        case dnnl_status_t::dnnl_invalid_graph:
+        case dnnl_status_t::dnnl_invalid_graph_op:
+        case dnnl_status_t::dnnl_invalid_shape:
+            return bdnn_state_t {
+                    res_state_t::SKIPPED, skip_reason::invalid_case};
+        case dnnl_status_t::dnnl_invalid_data_type:
+            return bdnn_state_t {
+                    res_state_t::SKIPPED, skip_reason::data_type_not_supported};
+        default: assert(!"dnnl state is not found!"); return bdnn_state_t {};
+    }
+}
+
 void compiled_partition_executor(dnnl::graph::compiled_partition &cp,
         dnnl::stream &stream, const std::vector<dnnl::graph::tensor> &inputs,
         const std::vector<dnnl::graph::tensor> &outputs) {
@@ -43,8 +70,11 @@ void compiled_partition_executor(dnnl::graph::compiled_partition &cp,
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
         dnnl::graph::sycl_interop::execute(cp, stream, inputs,
                 const_cast<std::vector<dnnl::graph::tensor> &>(outputs));
+#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
+        dnnl::graph::ocl_interop::execute(cp, stream, inputs,
+                const_cast<std::vector<dnnl::graph::tensor> &>(outputs));
 #else
-        assert(!"GPU only support DPCPP runtime now");
+        assert(!"unsupported gpu runtime");
 #endif
     }
 }
@@ -58,8 +88,8 @@ int execute_and_wait(const std::vector<dnnl::graph::compiled_partition> &cp_v,
         perf_function_t perf_func = std::bind(&compiled_partition_executor,
                 cp_v[i], std::placeholders::_1, std::placeholders::_2,
                 std::placeholders::_3);
-        DNN_GRAPH_SAFE(perf_func(stream, inputs_v[i], outputs_v[i]), CRIT);
-        DNN_GRAPH_SAFE(stream.wait(), CRIT);
+        DNN_GRAPH_SAFE(perf_func(stream, inputs_v[i], outputs_v[i]), CRIT, res);
+        DNN_GRAPH_SAFE(stream.wait(), CRIT, res);
     }
     res->state = EXECUTED;
     return OK;
@@ -77,7 +107,8 @@ inline dnnl::stream::flags get_profiling_flags() {
 inline int measure_perf_aggregate(timer::timer_t &t,
         std::vector<perf_function_t> &perf_func_v,
         const std::vector<std::vector<dnnl::graph::tensor>> &inputs_v,
-        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v) {
+        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v,
+        res_t *res) {
     const int max_batch_times = 4096;
     // Nvidia/AMD don't support profiling.
     const bool use_profiling = is_gpu() && !is_nvidia_gpu() && !is_amd_gpu();
@@ -90,8 +121,9 @@ inline int measure_perf_aggregate(timer::timer_t &t,
     // kernel has not been built and skews the results.
     auto sz = perf_func_v.size();
     for (size_t i = 0; i < sz; i++) {
-        DNN_GRAPH_SAFE(perf_func_v[i](stream, inputs_v[i], outputs_v[i]), WARN);
-        DNN_GRAPH_SAFE(stream.wait(), WARN);
+        DNN_GRAPH_SAFE(
+                perf_func_v[i](stream, inputs_v[i], outputs_v[i]), WARN, res);
+        DNN_GRAPH_SAFE(stream.wait(), WARN, res);
     }
 
     int cur_batch_times
@@ -105,10 +137,10 @@ inline int measure_perf_aggregate(timer::timer_t &t,
     while (true) {
         for_(int i = 0; i < cur_batch_times; i++)
         for (size_t j = 0; j < sz; j++) {
-            DNN_GRAPH_SAFE(
-                    perf_func_v[j](stream, inputs_v[j], outputs_v[j]), WARN);
+            DNN_GRAPH_SAFE(perf_func_v[j](stream, inputs_v[j], outputs_v[j]),
+                    WARN, res);
         }
-        DNN_GRAPH_SAFE(stream.wait(), WARN);
+        DNN_GRAPH_SAFE(stream.wait(), WARN, res);
 
         if (use_profiling) {
             std::vector<uint64_t> nsecs;
@@ -159,7 +191,8 @@ inline int measure_perf_aggregate(timer::timer_t &t,
 inline int measure_perf_individual(timer::timer_t &t,
         std::vector<perf_function_t> &perf_func_v,
         const std::vector<std::vector<dnnl::graph::tensor>> &inputs_v,
-        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v) {
+        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v,
+        res_t *res) {
     const bool use_profiling = is_gpu() && !is_nvidia_gpu() && !is_amd_gpu();
     const dnnl::stream::flags flags = use_profiling
             ? dnnl::stream::flags::default_flags | get_profiling_flags()
@@ -170,8 +203,8 @@ inline int measure_perf_individual(timer::timer_t &t,
     while (true) {
         auto sz = perf_func_v.size();
         for (size_t i = 0; i < sz; i++) {
-            DNN_GRAPH_SAFE(
-                    perf_func_v[i](stream, inputs_v[i], outputs_v[i]), WARN);
+            DNN_GRAPH_SAFE(perf_func_v[i](stream, inputs_v[i], outputs_v[i]),
+                    WARN, res);
         }
         t.stamp();
         if (should_stop(t)) break;
@@ -181,14 +214,17 @@ inline int measure_perf_individual(timer::timer_t &t,
 
 int measure_perf(timer::timer_t &t, std::vector<perf_function_t> &perf_func_v,
         const std::vector<std::vector<dnnl::graph::tensor>> &inputs_v,
-        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v) {
+        const std::vector<std::vector<dnnl::graph::tensor>> &outputs_v,
+        res_t *res) {
     if (has_bench_mode_bit(mode_bit_t::perf)) {
         // enable GPU profiling, Nvidia/AMD dose not support profiling.
         int ret = OK;
         if (is_cpu() && !is_sycl_engine()) {
-            ret = measure_perf_individual(t, perf_func_v, inputs_v, outputs_v);
+            ret = measure_perf_individual(
+                    t, perf_func_v, inputs_v, outputs_v, res);
         } else {
-            ret = measure_perf_aggregate(t, perf_func_v, inputs_v, outputs_v);
+            ret = measure_perf_aggregate(
+                    t, perf_func_v, inputs_v, outputs_v, res);
         }
         return ret;
     } else {
@@ -208,108 +244,12 @@ int measure_perf(timer::timer_t &t,
                 std::placeholders::_3));
     }
 
-    int status = measure_perf(t, perf_func_v, inputs_v, outputs_v);
+    int status = measure_perf(t, perf_func_v, inputs_v, outputs_v, res);
     if (res) res->state = EXECUTED;
 
     return status;
 }
 
-#ifdef DNNL_WITH_SYCL
-void *scratchpad_mm_mgr::sycl_alloc_mm(
-        size_t size, size_t alignment, const void *dev, const void *ctx) {
-    // fake malloc for 0 size
-    if (size == 0) return nullptr;
-
-    void *ptr {nullptr};
-    bool need_alloc_new_mm = true;
-    // find alloc mm with same size
-    const auto cnt = map_size_ptr_.count(size);
-    if (cnt > 0) {
-        const auto Iter = map_size_ptr_.equal_range(size);
-        for (auto it = Iter.first; it != Iter.second; ++it) {
-            // check if same size mm is free
-            if (free_ptr_.find(it->second.get()) != free_ptr_.end()) {
-                ptr = it->second.get();
-                free_ptr_.erase(ptr);
-                need_alloc_new_mm = false;
-            }
-        }
-    }
-
-    if (need_alloc_new_mm) {
-        auto sh_ptr = std::shared_ptr<void> {
-                malloc_shared(size, *static_cast<const sycl::device *>(dev),
-                        *static_cast<const sycl::context *>(ctx)),
-                sycl_deletor {*static_cast<const sycl::context *>(ctx)}};
-        ptr = sh_ptr.get();
-        // record the map of mm size and its ptr for reuse
-        map_size_ptr_.emplace(std::make_pair(size, sh_ptr));
-    }
-    return ptr;
-}
-
-void scratchpad_mm_mgr::sycl_free_mm(
-        void *ptr, const void *device, const void *context, void *event) {
-    free_ptr_.insert(ptr);
-}
-
-static scratchpad_mm_mgr s_mm_mgr;
-
-void *test_sycl_malloc_wrapper(
-        size_t n, size_t alignment, const void *dev, const void *ctx) {
-    return malloc_device(n, *static_cast<const sycl::device *>(dev),
-            *static_cast<const sycl::context *>(ctx));
-}
-
-void test_sycl_free_wrapper(
-        void *ptr, const void *dev, const void *context, void *event) {
-    (void)(dev);
-    if (event) {
-        static_cast<sycl::event *>(const_cast<void *>(event))->wait();
-    }
-    free(ptr, *static_cast<const sycl::context *>(context));
-}
-
-void *sycl_malloc_wrapper(
-        size_t size, size_t alignment, const void *dev, const void *ctx) {
-    void *ptr = has_bench_mode_bit(mode_bit_t::corr) || is_cpu()
-            ? test_sycl_malloc_wrapper(size, alignment, dev, ctx)
-            : s_mm_mgr.sycl_alloc_mm(size, alignment, dev, ctx);
-
-    return ptr;
-}
-
-// perf mode, mem will be finally released in s_mm_mgr ~shared_ptr when
-// test finished.
-void sycl_free_wrapper(
-        void *ptr, const void *device, const void *context, void *event) {
-    if (has_bench_mode_bit(mode_bit_t::corr) || is_cpu()) {
-        test_sycl_free_wrapper(ptr, device, context, event);
-    } else {
-        s_mm_mgr.sycl_free_mm(ptr, device, context, event);
-    }
-}
-
-sycl::queue &get_queue() {
-    static dnnl::engine test_eng {::get_test_engine()};
-    static sycl::device dev {dnnl::sycl_interop::get_device(test_eng)};
-    static sycl::context ctx {dnnl::sycl_interop::get_context(test_eng)};
-    static sycl::queue q {ctx, dev, sycl::property::queue::in_order {}};
-    return q;
-}
-#endif // DNNL_WITH_SYCL
-
-bool is_sycl_engine() {
-#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_SYCL
-    if (is_cpu()) return true;
-#endif
-
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-    if (!is_cpu()) return true;
-#endif
-    return false;
-}
-
 dnnl::graph::op::kind opstr2kind(const std::string &kind) {
     const std::unordered_map<std::string, dnnl::graph::op::kind> op_map = {
             {"Abs", dnnl::graph::op::kind::Abs},
@@ -347,6 +287,7 @@ dnnl::graph::op::kind opstr2kind(const std::string &kind) {
             {"Exp", dnnl::graph::op::kind::Exp},
             {"GELU", dnnl::graph::op::kind::GELU},
             {"GELUBackward", dnnl::graph::op::kind::GELUBackward},
+            {"GroupNorm", dnnl::graph::op::kind::GroupNorm},
             {"HardSigmoid", dnnl::graph::op::kind::HardSigmoid},
             {"HardSigmoidBackward", dnnl::graph::op::kind::HardSigmoidBackward},
             {"HardSwish", dnnl::graph::op::kind::HardSwish},
@@ -407,8 +348,9 @@ dnnl::graph::op::kind opstr2kind(const std::string &kind) {
     } else {
         fprintf(stderr, "graph: ERROR: Unsupported opkind: `%s`, exiting...\n",
                 kind.c_str());
-        exit(2);
+        SAFE_V(FAIL);
     }
+    return dnnl::graph::op::kind::LastSymbol;
 }
 
 dnnl::graph::op::attr attrstr2kind(const std::string &attr_name) {
@@ -475,8 +417,9 @@ dnnl::graph::op::attr attrstr2kind(const std::string &attr_name) {
         fprintf(stderr,
                 "graph: ERROR: Unsupported attribute: `%s`, exiting...\n",
                 attr_name.c_str());
-        exit(2);
+        SAFE_V(FAIL);
     }
+    return dnnl::graph::op::attr::undef;
 }
 
 class op_kind_hash_t {
@@ -536,6 +479,7 @@ dnnl_driver_t opkind2driver(const dnnl::graph::op::kind &kind) {
                     {dnnl::graph::op::kind::GELU, dnnl_driver_t::eltwise},
                     {dnnl::graph::op::kind::GELUBackward,
                             dnnl_driver_t::eltwise},
+                    {dnnl::graph::op::kind::GroupNorm, dnnl_driver_t::gnorm},
                     {dnnl::graph::op::kind::HardSigmoid,
                             dnnl_driver_t::eltwise},
                     {dnnl::graph::op::kind::HardSigmoidBackward,
@@ -621,48 +565,18 @@ dnnl_driver_t opkind2driver(const dnnl::graph::op::kind &kind) {
     } else {
         fprintf(stderr, "graph: ERROR: Unsupported opkind: `%d`, exiting...\n",
                 static_cast<int>(kind));
-        exit(2);
+        SAFE_V(FAIL);
     }
+    return dnnl_driver_t::others;
 }
 
-bool is_nxc_lt_arg(const std::string &kind, const int exec_arg) {
-    // Mapping from the op kind to a set that indicates which input arg needs
-    // reorder
-    static const std::unordered_map<std::string, std::unordered_set<int>>
-            input_arg_for_reorder = {
-                    {"AvgPool", {DNNL_ARG_SRC}},
-                    {"AvgPoolBackward", {DNNL_ARG_DIFF_DST}},
-                    {"BatchNormInference", {DNNL_ARG_SRC}},
-                    {"BatchNormForwardTraining", {DNNL_ARG_SRC}},
-                    {"BiasAddBackward", {DNNL_ARG_SRC}},
-                    {"Interpolate", {DNNL_ARG_SRC}},
-                    {"MaxPool", {DNNL_ARG_SRC}},
-                    {"Convolution", {DNNL_ARG_SRC}},
-                    {"ConvolutionBackwardData", {DNNL_ARG_DIFF_DST}},
-                    {"ConvTranspose", {DNNL_ARG_SRC}},
-                    {"ConvTransposeBackwardData", {DNNL_ARG_DIFF_DST}},
-                    {"BatchNormTrainingBackward",
-                            {DNNL_ARG_SRC, DNNL_ARG_DIFF_DST}},
-                    {"BiasAdd", {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1}},
-                    {"InterpolateBackward", {DNNL_ARG_DIFF_DST}},
-                    {"MaxPoolBackward", {DNNL_ARG_SRC, DNNL_ARG_DIFF_DST}},
-                    {"ConvolutionBackwardWeights",
-                            {DNNL_ARG_SRC, DNNL_ARG_DIFF_DST}},
-                    {"ConvTransposeBackwardWeights",
-                            {DNNL_ARG_SRC, DNNL_ARG_DIFF_DST}},
-                    {"PReLU", {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS}},
-                    {"PReLUBackward",
-                            {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS,
-                                    DNNL_ARG_DIFF_DST}},
-            };
-
-    const auto iter = input_arg_for_reorder.find(kind);
-    if (iter != input_arg_for_reorder.end()) {
-        const auto &args_to_reorder = iter->second;
-        return args_to_reorder.find(exec_arg) != args_to_reorder.end();
-    } else {
-        return false;
+// when length is 3, return "abc", when length is 5, return "abcde"
+std::string get_default_tag(size_t length) {
+    std::string mtag;
+    for (size_t i = 0; i < length; ++i) {
+        mtag += char('a' + i);
     }
+    return mtag;
 }
 
 std::string strides2memory_tag(const size_t ndims,
@@ -757,6 +671,25 @@ void change_format_to_ncx(dims_t &dims) {
     dims.erase(dims.end() - 1);
 }
 
+std::string verbose_partitions_n_ops(
+        const std::vector<dnnl::graph::partition> &partitions) {
+    std::string s;
+    for (const auto &partition : partitions) {
+        s += " {" + std::to_string(partition.get_ops_num()) + "}";
+    }
+    return s;
+}
+
+std::string lt_dims2str(const dnnl::graph::logical_tensor::dims &dims) {
+    if (dims.empty()) return std::string();
+
+    std::stringstream ss;
+    std::copy(
+            dims.begin(), dims.end(), std::ostream_iterator<int64_t>(ss, "x"));
+    auto res = ss.str();
+    return res.substr(0, res.length() - 1);
+}
+
 void permute_md(dnn_mem_t &mem, std::vector<int64_t> permutation) {
     std::vector<int> axes(permutation.size(), -1);
     for (int i = 0; i < static_cast<int>(permutation.size()); i++) {
@@ -771,29 +704,6 @@ void permute_md(dnn_mem_t &mem, std::vector<int64_t> permutation) {
     assert(st == dnnl_success);
     mem.md_ = clone_md(permuted_md.get());
 }
-void reshape_md(dnn_mem_t &mem, const dnnl::memory::dims &reshaped_dims,
-        const dnnl::memory::dims &reshaped_strides) {
-
-    const auto data_type = static_cast<dnnl::memory::data_type>(mem.dt());
-    dnnl::memory::desc md(reshaped_dims, data_type, reshaped_strides);
-    // Dirty hack to replace md with another one.
-    // TODO: replace it with a better solution.
-    auto st = dnnl_memory_desc_destroy(mem.md_);
-    (void)st;
-    assert(st == dnnl_success);
-    mem.md_ = clone_md(md.get());
-}
-
-void reshape_md(dnn_mem_t &mem, const dnnl::memory::dims &reshaped_dims) {
-    dnnl::memory::desc md(clone_md(mem.md_));
-    dnnl::memory::desc reshaped_md = md.reshape(reshaped_dims);
-    // Dirty hack to replace md with another one.
-    // TODO: replace it with a better solution.
-    auto st = dnnl_memory_desc_destroy(mem.md_);
-    (void)st;
-    assert(st == dnnl_success);
-    mem.md_ = clone_md(reshaped_md.get());
-}
 
 int get_prim_arg_name_from_graph_op_output_offset(
         dnnl::graph::op::kind op_kind, size_t output_offset) {
@@ -894,6 +804,21 @@ int get_prim_arg_name_from_graph_op_output_offset(
                 return -1;
             }
         } break;
+        case dnnl::graph::op::kind::GroupNorm: {
+            if (output_offset == 0)
+                return DNNL_ARG_DST;
+            else if (output_offset == 1)
+                return DNNL_ARG_MEAN;
+            else if (output_offset == 2)
+                return DNNL_ARG_VARIANCE;
+            else {
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                        static_cast<int>(output_offset));
+                assert(false);
+                return -1;
+            }
+
+        } break;
         default: {
             return DNNL_ARG_DST;
         } break;
@@ -901,7 +826,7 @@ int get_prim_arg_name_from_graph_op_output_offset(
 }
 
 int get_prim_arg_name_from_graph_op_input_offset(
-        dnnl::graph::op::kind op_kind, int input_offset, bool use_dst) {
+        dnnl::graph::op::kind op_kind, size_t input_offset, bool use_dst) {
     switch (op_kind) {
         case dnnl::graph::op::kind::Add:
         case dnnl::graph::op::kind::BiasAdd:
@@ -915,14 +840,14 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_SRC_1;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
             }
         }
         case dnnl::graph::op::kind::Concat: {
-            return DNNL_ARG_MULTIPLE_SRC + input_offset;
+            return DNNL_ARG_MULTIPLE_SRC + static_cast<int>(input_offset);
         } break;
         case dnnl::graph::op::kind::Convolution:
         case dnnl::graph::op::kind::ConvTranspose:
@@ -934,7 +859,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_BIAS;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -947,7 +872,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_WEIGHTS;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -960,7 +885,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_DIFF_DST;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -972,7 +897,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_WEIGHTS;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -986,7 +911,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_DIFF_DST;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -997,12 +922,12 @@ int get_prim_arg_name_from_graph_op_input_offset(
                 return DNNL_ARG_DIFF_DST;
             else if (input_offset == 1) {
                 BENCHDNN_PRINT(0,
-                        "Error: no support for input %d of Avg Pool Backward",
+                        "Error: no support for input %zu of Avg Pool Backward",
                         input_offset);
                 assert(false);
                 return -1;
             } else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1015,12 +940,12 @@ int get_prim_arg_name_from_graph_op_input_offset(
                 return DNNL_ARG_DIFF_DST;
             else if (input_offset == 2) {
                 BENCHDNN_PRINT(0,
-                        "Error: no support for input %d of Max Pool Backward",
+                        "Error: no support for input %zu of Max Pool Backward",
                         input_offset);
                 assert(false);
                 return -1;
             } else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1033,7 +958,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_DST;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1047,7 +972,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_SHIFT;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1067,7 +992,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 5)
                 return DNNL_ARG_SHIFT;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1085,7 +1010,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 4)
                 return DNNL_ARG_SHIFT;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1103,7 +1028,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 4)
                 return DNNL_ARG_VARIANCE;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1121,7 +1046,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 4)
                 return DNNL_ARG_SCALE;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1134,12 +1059,13 @@ int get_prim_arg_name_from_graph_op_input_offset(
                 return DNNL_ARG_DIFF_DST;
             else if (input_offset == 2) {
                 BENCHDNN_PRINT(0,
-                        "Error: no support for input %d of Resampling Backward",
+                        "Error: no support for input %zu of Resampling "
+                        "Backward",
                         input_offset);
                 assert(false);
                 return -1;
             } else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1155,12 +1081,13 @@ int get_prim_arg_name_from_graph_op_input_offset(
             if (input_offset == 0)
                 return DNNL_ARG_SRC;
             else if (input_offset == 1) {
-                BENCHDNN_PRINT(0, "Error: no support for input %d of Reduction",
+                BENCHDNN_PRINT(0,
+                        "Error: no support for input %zu of Reduction",
                         input_offset);
                 assert(false);
                 return -1;
             } else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1183,7 +1110,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 1)
                 return DNNL_ARG_DIFF_DST;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1203,7 +1130,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_FROM;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1217,7 +1144,7 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_TO;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1231,7 +1158,21 @@ int get_prim_arg_name_from_graph_op_input_offset(
             else if (input_offset == 2)
                 return DNNL_ARG_SRC_1;
             else {
-                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %d",
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
+                        input_offset);
+                assert(false);
+                return -1;
+            }
+        } break;
+        case dnnl::graph::op::kind::GroupNorm: {
+            if (input_offset == 0)
+                return DNNL_ARG_SRC;
+            else if (input_offset == 1)
+                return DNNL_ARG_SCALE;
+            else if (input_offset == 2)
+                return DNNL_ARG_SHIFT;
+            else {
+                BENCHDNN_PRINT(0, "Error: no matching ARG for offset %zu",
                         input_offset);
                 assert(false);
                 return -1;
@@ -1270,26 +1211,53 @@ cpp_stream_t::cpp_stream_t(
 }
 
 cpp_engine_t::cpp_engine_t() {
+
+    dnnl::graph::allocator &alloc = get_graph_allocator();
+
     if (is_cpu()) {
-#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_SYCL
-        static dnnl::graph::allocator alloc {
-                dnnl::graph::sycl_interop::make_allocator(
-                        sycl_malloc_wrapper, sycl_free_wrapper)};
-#else
-        static dnnl::graph::allocator alloc {};
-#endif
         engine_ = make_engine_with_allocator(dnnl::engine::kind::cpu,
                 static_cast<size_t>(engine_index), alloc);
     } else {
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        static dnnl::graph::allocator alloc {
-                dnnl::graph::sycl_interop::make_allocator(
-                        sycl_malloc_wrapper, sycl_free_wrapper)};
         engine_ = make_engine_with_allocator(dnnl::engine::kind::gpu,
                 static_cast<size_t>(engine_index), alloc);
+#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
+        // needs to prepare ocl_malloc_wrapper and ocl_free_wrapper and call
+        // make_engine_with_allocator instead.
+        engine_ = dnnl::engine(
+                dnnl::engine::kind::gpu, static_cast<size_t>(engine_index));
 #else
-        assert(!"GraphAPI GPU only support DPCPP runtime now");
+        assert(!"unsupported gpu runtime");
 #endif
     }
 }
+
+bool is_gc_backend() {
+#if defined(ENABLE_GRAPH_COMPILER_BACKEND)
+    return true;
+#else
+    return false;
+#endif
+}
+
+dnnl_data_type_t convert_dt(const dnnl::graph::logical_tensor::data_type dt) {
+    using graph_dt = dnnl::graph::logical_tensor::data_type;
+
+    switch (dt) {
+        case graph_dt::f16: return dnnl_f16;
+        case graph_dt::bf16: return dnnl_bf16;
+        case graph_dt::f32: return dnnl_f32;
+        case graph_dt::s32: return dnnl_s32;
+        case graph_dt::s8: return dnnl_s8;
+        case graph_dt::u8: return dnnl_u8;
+        // use u8 instead of boolean in the reference path
+        // dnn_graph_mem_t will use the data type from the logical tensor and the u8 data handle
+        case graph_dt::boolean: return dnnl_u8;
+        case graph_dt::f8_e5m2: return dnnl_f8_e5m2;
+        case graph_dt::f8_e4m3: return dnnl_f8_e4m3;
+        case graph_dt::undef:
+        default: return dnnl_data_type_undef;
+    }
+}
+
 } // namespace graph
