@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2023 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -40,6 +40,11 @@ int fill_mem(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
     const auto nelems = mem_fp.nelems();
     if (nelems == 0) return OK;
 
+    // Refer to modes documentation for filling principles.
+    if (has_bench_mode_bit(mode_bit_t::bitwise)) {
+        return fill_random_real(mem_dt, mem_fp, nullptr);
+    }
+
     const auto conf = prb->get_conf(kind);
 
     const float gen[] = {
@@ -135,15 +140,16 @@ int compare_compensation(const prb_t *prb, dnn_mem_map_t &mem_map,
 dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
     const prb_t *prb = init_pd_args.prb;
     res_t *res = init_pd_args.res;
+    bool force_f32_dt = init_pd_args.force_f32_dt;
 
     auto dims = prb->dims;
     for (int d = 0; d < prb->ndims; ++d)
         if (prb->runtime_dim_mask & (1 << d)) dims[d] = DNNL_RUNTIME_DIM_VAL;
 
-    auto src_d
-            = dnn_mem_t::init_md(prb->ndims, dims.data(), prb->sdt, prb->stag);
-    auto dst_d
-            = dnn_mem_t::init_md(prb->ndims, dims.data(), prb->ddt, prb->dtag);
+    auto src_d = dnn_mem_t::init_md(prb->ndims, dims.data(),
+            force_f32_dt ? dnnl_f32 : prb->sdt, prb->stag, prb->strides[0]);
+    auto dst_d = dnn_mem_t::init_md(prb->ndims, dims.data(),
+            force_f32_dt ? dnnl_f32 : prb->ddt, prb->dtag, prb->strides[1]);
 
     // Prepare and assign extra for dst_md.
     auto &extra = static_cast<dnnl_memory_desc_t>(dst_d)->extra;
@@ -214,7 +220,8 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
     }
 #endif
     if (!scales_ok) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 
@@ -239,14 +246,16 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
         }
 
         if (!dt_ok || !attr_ok || !rt_ok || !masks_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
 
 #if !defined(DNNL_X64) || DNNL_X64 == 0
         // Simple reorder doesn't provide decent coverage for compensated cases.
         // Shut them down unconditionally by default.
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
 #endif
     }
@@ -257,7 +266,8 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
     const auto &dst_scales = prb->attr.scales.get(DNNL_ARG_DST);
     if (!dst_scales.is_def() && attr_t::get_default_mask(dst_scales.policy) > 0
             && prb->runtime_dim_mask != 0) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 
@@ -268,17 +278,31 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
         if (attr_t::get_default_mask(src_scales.policy)
                         != attr_t::get_default_mask(dst_scales.policy)
                 && prb->is_reorder_with_compensation(FLAG_ANY)) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
 
     if (is_cpu()) {
-        // CPU reorder doesn't support bf16<-->s32 combinations.
-        const bool s32_src_ok = IMPLICATION(sdt == dnnl_s32, ddt != dnnl_bf16);
-        const bool s32_dst_ok = IMPLICATION(ddt == dnnl_s32, sdt != dnnl_bf16);
+        // Int4 reorder support is limited on CPU.
+        if (sdt == dnnl_s4 || ddt == dnnl_s4 || sdt == dnnl_u4
+                || ddt == dnnl_u4) {
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+        }
+
+        // CPU reorder doesn't support (xf8,xf16)<-->s32 combinations.
+        const bool s32_src_ok = IMPLICATION(sdt == dnnl_s32,
+                ddt != dnnl_f8_e5m2 && ddt != dnnl_f8_e4m3 && ddt != dnnl_bf16
+                        && ddt != dnnl_f16);
+        const bool s32_dst_ok = IMPLICATION(ddt == dnnl_s32,
+                sdt != dnnl_f8_e5m2 && sdt != dnnl_f8_e4m3 && sdt != dnnl_bf16
+                        && sdt != dnnl_f16);
         if (!s32_src_ok || !s32_dst_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
 
@@ -288,7 +312,21 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
         const bool f16_dst_ok = IMPLICATION(
                 ddt == dnnl_f16, sdt == dnnl_f16 || sdt == dnnl_f32);
         if (!f16_src_ok || !f16_dst_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+        }
+
+        // CPU xf8 reorders only support xf8<->(f16,f32) combinations
+        const bool xf8_src_ok
+                = IMPLICATION(ddt == dnnl_f8_e5m2 || ddt == dnnl_f8_e4m3,
+                        sdt == dnnl_f16 || sdt == dnnl_f32);
+        const bool xf8_dst_ok
+                = IMPLICATION(sdt == dnnl_f8_e5m2 || sdt == dnnl_f8_e4m3,
+                        ddt == dnnl_f16 || ddt == dnnl_f32);
+        if (!xf8_src_ok || !xf8_dst_ok) {
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -299,7 +337,17 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
         // in kernels directly, but s8s8 instructions are available in HW.
         if (prb->runtime_dim_mask != 0
                 || prb->is_reorder_with_compensation(FLAG_ANY)) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+        }
+
+        // GPU doesn't support f8_e5m2/f8_e4m3.
+        const bool is_xf8 = prb->sdt == dnnl_f8_e5m2 || prb->sdt == dnnl_f8_e4m3
+                || prb->ddt == dnnl_f8_e5m2 || prb->ddt == dnnl_f8_e4m3;
+        if (is_xf8) {
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -310,14 +358,18 @@ void skip_invalid_prb(const prb_t *prb, res_t *res) {
 #if DNNL_CPU_RUNTIME == DNNL_RUNTIME_NONE \
         || DNNL_GPU_RUNTIME == DNNL_RUNTIME_NONE
     auto cross_engine = prb->cross_engine;
-    if (cross_engine == CPU2GPU || cross_engine == GPU2CPU)
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+    if (cross_engine == CPU2GPU || cross_engine == GPU2CPU) {
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
+        return;
+    }
 #endif
 
     // Zero-points can't be used with sum post-op.
     if (!prb->attr.zero_points.is_def(DNNL_ARG_DST)
             && prb->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM) != -1) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 
@@ -327,7 +379,8 @@ void skip_invalid_prb(const prb_t *prb, res_t *res) {
     const bool is_dst_zp_ok = is_integral_dt(prb->ddt)
             || prb->attr.zero_points.is_def(DNNL_ARG_DST);
     if (!(is_src_zp_ok && is_dst_zp_ok)) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 }
@@ -338,6 +391,9 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
     // Avoid any scales logic involved until needed.
     cmp.set_zero_trust_percent(80.f);
 
+    // `f8_e4m3` range is very short which makes inputs convert into NaNs.
+    cmp.set_op_output_has_nans(prb->sdt == dnnl_f8_e4m3);
+
     // Additional check to avoid false-positive result from f32->s32 conversion
     // in case of sum post-op on GPU happening when two max_dt values
     // are summed together.
@@ -362,7 +418,7 @@ std::vector<int> supported_exec_args(dir_t dir) {
 };
 
 int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
-        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res,
         dnnl_primitive_t prim_ref) {
     if (has_bench_mode_modifier(mode_modifier_t::no_host_memory)) return OK;
 
@@ -370,10 +426,19 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
 
     for (auto &entry : mem_map) {
         const int exec_arg = entry.first;
+        // The function targets regular exec_args that are positive.
+        // Negative args are used by bitwise and are broken in the `default`
+        // branch due to `&` always returns `true`.
+        if (exec_arg <= 0) continue;
+
         auto &mem = entry.second; // `mem` is modified by filler (reorder).
 
-        ref_mem_map.emplace(
-                exec_arg, dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+        // Scratchpad memory relates to a primitive. If reference needs it,
+        // use switch below to define a memory desc for it.
+        if (exec_arg != DNNL_ARG_SCRATCHPAD) {
+            ref_mem_map.emplace(exec_arg,
+                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+        }
         auto &ref_mem = ref_mem_map[exec_arg];
 
         switch (exec_arg) {
@@ -391,24 +456,19 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
                 // MIOpen doesn't work properly when tensors are filled with 0xFF.
                 if (sum_idx >= 0 || is_amd_gpu()) {
                     SAFE(fill_mem(prb, DST, mem, ref_mem), WARN);
-                }
-            } break;
-            case DNNL_ARG_SCRATCHPAD: break;
-            default: { // Process all attributes here
-                bool is_scales_arg = (exec_arg & DNNL_ARG_ATTR_SCALES);
-                bool is_zero_point_arg = (exec_arg & DNNL_ARG_ATTR_ZERO_POINTS);
 
-                if (is_scales_arg) {
-                    int local_exec_arg = exec_arg ^ DNNL_ARG_ATTR_SCALES;
-                    SAFE(fill_scales(prb->attr, local_exec_arg, mem, ref_mem),
-                            WARN);
-                } else if (is_zero_point_arg) {
-                    int local_exec_arg = exec_arg ^ DNNL_ARG_ATTR_ZERO_POINTS;
-                    SAFE(fill_zero_points(
-                                 prb->attr, local_exec_arg, mem, ref_mem),
-                            WARN);
+                    // Bitwise mode for sum requires a copy due to data for
+                    // post-op will be overwritten and it must be refreshed.
+                    if (has_bench_mode_bit(mode_bit_t::bitwise)) {
+                        SAFE(mem_map.at(-exec_arg).reorder(ref_mem), WARN);
+                    }
                 }
             } break;
+            default:
+                SAFE(init_ref_memory_args_default_case(
+                             exec_arg, mem, ref_mem, prb->attr, res),
+                        WARN);
+                break;
         }
         // Don't keep reference memory if it is not used further.
         if (!has_bench_mode_bit(mode_bit_t::corr)) ref_mem_map.clear();
@@ -436,9 +496,8 @@ int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
 
     dnn_mem_map_t mem_map, ref_mem_map;
     init_memory_args<prb_t>(mem_map, prb, prim, supported_exec_args(prb->dir));
-    TIME_FILL(SAFE(init_ref_memory_args(
-                           ref_mem_map, mem_map, prim, prb, res, prb->dir),
-            WARN));
+    TIME_FILL(SAFE(
+            init_ref_memory_args(ref_mem_map, mem_map, prim, prb, res), WARN));
 
     args_t args(mem_map), ref_args(ref_mem_map);
 
@@ -463,6 +522,7 @@ int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
             compare_compensation(prb, mem_map, ref_mem_map, res);
         }
     }
+    SAFE(check_bitwise(prim, {DST}, args, prb->attr, prb->inplace, res), WARN);
 
     return measure_perf(prb->ctx_exe, res, prim, args);
 }
