@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2023 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 *******************************************************************************/
 
 #include <algorithm>
-#include <atomic>
 #include <cmath>
+#include <mutex>
 #include <sstream>
 #include <string>
 
@@ -63,25 +63,38 @@ void dump_norm_values(
             diff_norm.diff_[norm_t::L8], diff_norm.rel_diff(norm_t::L8));
 }
 
-bool has_binary_comparison_po(const attr_t &attr) {
+bool has_binary_po_algs(const attr_t &attr,
+        const std::vector<attr_t::post_ops_t::kind_t> &algs) {
     const auto &po = attr.post_ops;
     if (po.is_def()) return false;
 
-    using alg_t = attr_t::post_ops_t::kind_t;
-    static const std::vector<alg_t> cmp_alg = {alg_t::MAX, alg_t::MIN,
-            alg_t::GE, alg_t::GT, alg_t::LE, alg_t::LT, alg_t::EQ, alg_t::NE};
-
     for (int idx = 0; idx < po.len(); ++idx) {
         const auto &e = po.entry[idx];
         if (!e.is_binary_kind()) continue;
 
-        if (std::any_of(cmp_alg.cbegin(), cmp_alg.cend(),
-                    [&](const alg_t alg) { return e.kind == alg; }))
+        if (std::any_of(algs.cbegin(), algs.cend(),
+                    [&](const attr_t::post_ops_t::kind_t alg) {
+                        return e.kind == alg;
+                    }))
             return true;
     }
     return false;
 }
 
+bool has_binary_comparison_po(const attr_t &attr) {
+    using alg_t = attr_t::post_ops_t::kind_t;
+    static const std::vector<alg_t> cmp_alg = {alg_t::MAX, alg_t::MIN,
+            alg_t::GE, alg_t::GT, alg_t::LE, alg_t::LT, alg_t::EQ, alg_t::NE};
+    return has_binary_po_algs(attr, cmp_alg);
+}
+
+bool has_binary_compute_po(const attr_t &attr) {
+    using alg_t = attr_t::post_ops_t::kind_t;
+    static const std::vector<alg_t> cmp_alg
+            = {alg_t::ADD, alg_t::SUB, alg_t::MUL, alg_t::DIV};
+    return has_binary_po_algs(attr, cmp_alg);
+}
+
 bool negative_converts_to_zero(const attr_t &attr, dnnl_data_type_t target_dt) {
     using po_kind_t = attr_t::post_ops_t::kind_t;
     const auto &po = attr.post_ops;
@@ -141,18 +154,31 @@ int compare_t::compare_norm(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
         return OK;
     }
 
-    res->total = nelems;
+    res->total = 1;
 
     dnn_mem_t got_f32(got_mem, dnnl_f32, tag::abx, get_cpu_engine());
     const auto dt = got_mem.dt();
 
+    // Idea is to pad nelems to mimic uniform load between available threads.
+    // It allows to make a clear assumption when the last element is processed
+    // and do a sync with global diff_norm object.
+    // The better alternative is to expose `parallel` interface.
+    const auto nthreads = benchdnn_get_max_threads();
+    const auto nelems_per_thread = div_up(nelems, nthreads);
+    const auto nelems_pad = nelems_per_thread * nthreads;
+
     diff_norm_t diff_norm;
     const bool need_dump = verbose >= 99;
-    for (int64_t i = 0; i < nelems; ++i) {
+
+    const auto compare_norm_values = [&](int64_t i) {
+        if (i >= nelems) return;
+
+        // Specifiers to keep data accumulated over several `i`.
+        static thread_local diff_norm_t diff_norm_ithr;
         driver_check_func_args_t args(exp_mem, got_f32, i, dt, trh_);
 
         if ((std::isnan(args.exp_f32) && is_integral_dt(dt))
-                || std::isinf(args.exp_f32)) {
+                || std::isinf(args.exp)) {
             // Don't include integer max values or inf values into norm as they
             // make it irrelevant for validation.
             ;
@@ -163,14 +189,32 @@ int compare_t::compare_norm(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
             // it irrelevant for validation.
             ;
         } else {
-            diff_norm.update(args.exp, args.got);
+            diff_norm_ithr.update(args.exp, args.got);
         }
 
-        if (need_dump)
+        // Synchronization point, exchange to main diff_norm and reset thread's
+        // diff_norm_ithr object.
+        if (((i + 1) % nelems_per_thread == 0) || (i == nelems - 1)) {
+            static std::mutex m;
+            std::lock_guard<std::mutex> guard(m);
+            diff_norm.update(diff_norm_ithr);
+            diff_norm_ithr = diff_norm_t();
+        }
+    };
+
+    // Parallel norm computation to speed up the process.
+    benchdnn_parallel_nd(nelems_pad, compare_norm_values);
+
+    diff_norm.done();
+
+    // Serial point dump with enabled dumping when needed for nicer output.
+    if (need_dump) {
+        for (int64_t i = 0; i < nelems; ++i) {
+            driver_check_func_args_t args(exp_mem, got_f32, i, dt, trh_);
             dump_point_values(got_mem.md_, get_kind_str(), i, args.exp_f32,
                     args.exp, args.got, args.diff, args.rel_diff);
+        }
     }
-    diff_norm.done();
 
     bool ok = diff_norm.rel_diff(norm_t::L2) <= trh_;
     if (!ok) res->errors = 1;
@@ -179,6 +223,12 @@ int compare_t::compare_norm(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
     if (dump) dump_norm_values(diff_norm, get_kind_str());
 
     if (res->errors) res->state = FAILED;
+
+    // Status may be propagated from previous tensor. Use stats from cur tensor.
+    BENCHDNN_PRINT((res->errors ? 0 : 6),
+            "[COMPARE_STATS]%s: trh=%g (compare against [L2] rel_diff)\n",
+            get_kind_str().c_str(), trh_);
+
     if (res->state == EXECUTED) res->state = PASSED;
 
     return res->state == FAILED ? FAIL : OK;
@@ -192,29 +242,64 @@ int compare_t::compare_p2p(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
         return OK;
     }
 
-    res->total = nelems;
+    res->total += nelems;
 
     dnn_mem_t got_f32(got_mem, dnnl_f32, tag::abx, get_cpu_engine());
+    dnn_mem_t exp_f32_plain;
+    if (has_prim_ref_
+            && !check_md_consistency_with_tag(exp_mem.md_, tag::abx)) {
+        exp_f32_plain
+                = dnn_mem_t(exp_mem, dnnl_f32, tag::abx, get_cpu_engine());
+    }
+    const dnn_mem_t &exp_f32 = exp_f32_plain ? exp_f32_plain : exp_mem;
+
     const auto dt = got_mem.dt();
     const bool has_eltwise
             = attr.post_ops.eltwise_index() != -1 || has_eltwise_post_op_;
+    const std::vector<dnnl_data_type_t> dt_with_nan {
+            dnnl_f16, dnnl_e8m0, dnnl_f8_e5m2, dnnl_f8_e4m3};
     const bool output_has_nans = op_output_has_nans_
             || eltwise::eltwise_alg_returns_nan_or_inf(attr)
-            || got_mem.dt() == dnnl_f16;
+            || std::any_of(dt_with_nan.begin(), dt_with_nan.end(),
+                    [&](dnnl_data_type_t dt) { return got_mem.dt() == dt; });
     const bool has_exp_eltwise
             = attr.post_ops.find(attr_t::post_ops_t::kind_t::EXP) >= 0;
     const bool has_dst_scale = !attr.scales.get(DNNL_ARG_DST).is_def();
 
-    // Atomics to be updated in parallel section, non-atomics - in sequential.
-    std::atomic<bool> all_ok(true);
-    std::atomic<int64_t> zeros(0);
-    std::atomic<float> max_rdiff(0), max_diff(0);
+    // Idea is to pad nelems to mimic uniform load between available threads.
+    // It allows to make a clear assumption when the last element is processed
+    // and do a sync with global diff_norm object.
+    // The better alternative is to expose `parallel` interface.
+    const auto nthreads = benchdnn_get_max_threads();
+    const auto nelems_per_thread = div_up(nelems, nthreads);
+    const auto nelems_pad = nelems_per_thread * nthreads;
+
+    // These global metrics are updated at the synchronization point.
+    bool global_ok = true;
+    int64_t zeros = 0;
+    // "all_" stuff is across the whole tensor. "err_" stuff is just for points
+    // that didn't pass any criteria.
+    float all_max_rdiff = 0.f;
+    float all_max_diff = 0.f;
+    float err_max_rdiff = 0.f;
+    float err_max_diff = 0.f;
     int64_t n_errors = 0;
     volatile bool from_parallel = true;
     const bool need_dump = verbose >= 99;
 
     const auto compare_point_values = [&](int64_t i) {
-        driver_check_func_args_t args(exp_mem, got_f32, i, dt, trh_);
+        // Skip padded (non-existent) elements.
+        if (i >= nelems) return;
+
+        // Stats for all validated points per one thread.
+        static thread_local bool ithr_ok = true;
+        static thread_local int64_t ithr_zeros = 0;
+        static thread_local float ithr_all_max_rdiff = 0.f;
+        static thread_local float ithr_all_max_diff = 0.f;
+        static thread_local float ithr_err_max_rdiff = 0.f;
+        static thread_local float ithr_err_max_diff = 0.f;
+
+        driver_check_func_args_t args(exp_f32, got_f32, i, dt, trh_);
 
         bool ok = args.diff == 0.f;
         if (std::isnan(args.exp_f32) && is_integral_dt(dt)) {
@@ -222,105 +307,178 @@ int compare_t::compare_p2p(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
             // may implement NaN fp32 -> int32 conversion in a different manner.
             ok = true;
         }
-        // If fast check failed, go through all of them.
-        if (!ok) {
-            // Standard check for relative diff is under set threshold...
+
+        for (int z = 0; !ok && z < 1; z++) {
+            // Standard check for relative diff is under set threshold.
             ok = (fabsf(args.exp) > 1e-5f ? args.rel_diff : args.diff) <= trh_;
-            // If not, when NaNs or infinity are allowed for the driver, check
-            // that both exp and got are NaNs or infinity with same sign...
-            if (!ok && output_has_nans)
-                ok = compare::compare_extreme_values(args.exp, args.got);
-            // If not, use hack to check not fully correct s32 saturation on
-            // cpu...
-            if (!ok && is_cpu() && dt == dnnl_s32
-                    && args.exp == max_dt(dnnl_s32))
-                ok = args.got >= BENCHDNN_S32_TO_F32_SAT_CONST
-                        && args.got < max_dt(dnnl_s32);
-            // If not, check driver additional checks if set...
-            if (!ok && driver_check_func_) ok = driver_check_func_(args);
-            // If not, check if there are eltwise post-ops, use very relaxed
+            if (ok) break;
+
+            // When NaNs or infinity are allowed for the driver, check
+            // that both exp and got are NaNs or infinity with same sign.
+            ok = output_has_nans
+                    && compare::compare_extreme_values(args.exp, args.got);
+            if (ok) break;
+
+            // Use hack to check not fully correct s32 saturation on CPU.
+            ok = is_cpu() && dt == dnnl_s32 && args.exp == max_dt(dnnl_s32)
+                    && args.got >= BENCHDNN_S32_TO_F32_SAT_CONST
+                    && args.got < max_dt(dnnl_s32);
+            if (ok) break;
+
+            // Check driver's additional checks, if set.
+            ok = driver_check_func_ && driver_check_func_(args);
+            if (ok) break;
+
+            // Check if there are eltwise post-ops, use very relaxed
             // comparison since we can't control inputs for each driver finely
             // or validate if the output value from operation satisfies the
             // check for catastrophic cancellation (see eltwise additional check
             // function). We rely on validation of pure eltwise and let some
             // big rdiff errors slip away hoping that absolute error is good
             // enough.
-            if (!ok && has_eltwise) {
-                const float experimental_tolerated_trh
-                        = std::max(epsilon_dt(dt), 2e-5f);
-                ok = args.diff <= experimental_tolerated_trh;
-            }
+            const float experimental_eltwise_trh
+                    = std::max(epsilon_dt(dt), 2e-5f);
+            ok = has_eltwise && args.diff <= experimental_eltwise_trh;
+            if (ok) break;
+
             // For eltwise it also may happen that threshold is really small,
             // but absolute difference is really big. Also exponent is a special
             // transcendental post-op that has accuracy issues with older isa.
-            if (!ok && has_eltwise
-                    && (fabsf(args.exp) > 1e+5f || has_exp_eltwise)) {
-                ok = args.rel_diff <= std::max(epsilon_dt(dt), 5e-6f);
-            }
+            ok = has_eltwise && (fabsf(args.exp) > 1e+5f || has_exp_eltwise)
+                    && args.rel_diff <= std::max(epsilon_dt(dt), 5e-6f);
+            if (ok) break;
+
             // Attr dst scale is used as a divisor to quantize data to dt.
             // Implementation might decide to pre-compute inverse value and
             // multiply on it in kernel. This difference might result in a
             // slight error comparing to a division operation.
-            if (!ok && has_dst_scale) {
-                const float experimental_tolerated_trh
-                        = std::max(epsilon_dt(dt), 1e-5f);
-                ok = args.rel_diff <= experimental_tolerated_trh;
-            }
+            const float experimental_dst_scale_trh
+                    = std::max(epsilon_dt(dt), 1e-5f);
+            ok = has_dst_scale && args.rel_diff <= experimental_dst_scale_trh;
+            if (ok) break;
+
             // Binary MAX, MIN and comparison operations post-ops may return
             // different results for different backends when NaN is one of
             // inputs. Depending on its position and implementation, either
             // first or second operand may be returned.
-            if (!ok && has_binary_comparison_po(attr) && output_has_nans)
-                ok = true;
+            ok = has_binary_comparison_po(attr) && output_has_nans;
+            if (ok) break;
+
+            // Binary Add/Sub/Mul/Div usually produce additional noise in the
+            // output. For Add/Sub it's mostly catastrophic cancellation, for
+            // Mul it's usually rounding, for Div it's usually different
+            // precision level of instructions for different backends. Those are
+            // hard to fix with filling adjustments. Since binary po filling
+            // operates with integers or large numbers, it's safe to let some
+            // minor diff error to exist under assumption that original problem
+            // lacks those errors.
+            //
+            // Note: use specific dt and correspondent values not to mess with
+            // broad set of supported data types.
+            float binary_comp_po_trh = 0.f;
+            if (args.dt == dnnl_f16)
+                binary_comp_po_trh = 5.f * epsilon_dt(args.dt); // == 5e-3;
+            if (args.dt == dnnl_f32)
+                binary_comp_po_trh = 20.f * epsilon_dt(args.dt); // == 2e-6f;
+            ok = has_binary_compute_po(attr) && args.diff <= binary_comp_po_trh;
+            if (ok) break;
+
             // Some drivers (like pooling or resampling) on integer data types
             // may result in sporadic order of operations. This may cause a
             // difference around `x.5f` value, and can be rounded either way to
             // `x` or `x + 1` which can't be fixed by filling.
-            if (!ok && is_integral_dt(args.dt)) {
+            //
+            // Another class of `off-by-1` issues coming from optimized
+            // reference when transcendental operation present in the chain. In
+            // such cases, there's no way to test original output as both
+            // outputs would be rounded to integer number.
+            const auto is_int8_round_good = [args]() -> bool {
                 // Check that original value is close to x.5f.
                 static constexpr float small_eps = 9e-6f;
                 const float floor_val = floorf(args.exp_f32);
                 const float ceil_val = ceilf(args.exp_f32);
-                if (fabsf((floor_val + 0.5f) - args.exp_f32) < small_eps) {
-                    // If it is, check exp and got values are on opposite sides.
-                    if (args.exp == floor_val) {
-                        ok = args.got == ceil_val;
-                    } else if (args.exp == ceil_val) {
-                        ok = args.got == floor_val;
-                    }
+                if (fabsf((floor_val + 0.5f) - args.exp_f32) >= small_eps)
+                    return false;
+
+                // If it is, check exp and got values are on opposite sides.
+                if (args.exp == floor_val) {
+                    return args.got == ceil_val;
+                } else if (args.exp == ceil_val) {
+                    return args.got == floor_val;
                 }
-            }
+                return false;
+            };
+            const auto is_int8_prim_ref_and_transcedental = [&]() -> bool {
+                if (!has_prim_ref_) return false;
+                if (fabsf(args.exp_f32 - args.got) != 1) return false;
+                // TODO: update with transcendental eltwise ops only.
+                return has_eltwise;
+            };
+            ok = is_integral_dt(args.dt)
+                    && (is_int8_round_good()
+                            || is_int8_prim_ref_and_transcedental());
+            if (ok) break;
+
             // Nvidia backend with fpmath mode enabled returns not exact output
             // values (presumably on conversion to fp32), thus, make sure they
             // fit single ulp for a reduced data type.
-            if (!ok && is_nvidia_gpu()
-                    && attr.fpmath_mode != dnnl_fpmath_mode_strict) {
-                const auto deduced_src_dt = deduce_cfg_data_type(dt, attr, SRC);
-                ok = args.diff <= epsilon_dt(deduced_src_dt);
-            }
-        }
-        // Update compare stats.
-        if (from_parallel && fabsf(args.got) == 0) zeros++;
-        if (from_parallel && verbose >= 6) {
-            max_rdiff = MAX2(max_rdiff.load(), args.rel_diff);
-            max_diff = MAX2(max_diff.load(), args.diff);
+            ok = is_nvidia_gpu()
+                    && attr.fpmath_mode.mode != dnnl_fpmath_mode_strict
+                    && args.diff
+                            <= epsilon_dt(deduce_cfg_data_type(dt, attr, SRC));
+            if (ok) break;
         }
 
-        if (!ok && all_ok) all_ok = false;
+        // Update compare stats.
+        if (from_parallel) {
+            if (fabsf(args.got) == 0) ithr_zeros++;
+            ithr_all_max_rdiff = MAX2(ithr_all_max_rdiff, args.rel_diff);
+            ithr_all_max_diff = MAX2(ithr_all_max_diff, args.diff);
+            if (!ok)
+                ithr_err_max_rdiff = MAX2(ithr_err_max_rdiff, args.rel_diff);
+            if (!ok) ithr_err_max_diff = MAX2(ithr_err_max_diff, args.diff);
+        }
+
+        if (!ok && ithr_ok) ithr_ok = false;
         if (!ok && !from_parallel) n_errors++;
 
         const bool dump
-                = need_dump || (!ok && (n_errors < 10 || verbose >= 10));
+                = need_dump || (!ok && (n_errors <= 10 || verbose >= 10));
         if (!from_parallel && dump)
             dump_point_values(got_mem.md_, get_kind_str(), i, args.exp_f32,
                     args.exp, args.got, args.diff, args.rel_diff);
+
+        // Synchronization point, update global stats from thread stats.
+        if (((i + 1) % nelems_per_thread == 0) || (i == nelems - 1)) {
+            static std::mutex m;
+            std::lock_guard<std::mutex> guard(m);
+
+            if (global_ok && !ithr_ok) global_ok = false;
+            ithr_ok = true;
+
+            if (from_parallel) {
+                zeros += ithr_zeros;
+                all_max_rdiff = MAX2(all_max_rdiff, ithr_all_max_rdiff);
+                all_max_diff = MAX2(all_max_diff, ithr_all_max_diff);
+                err_max_rdiff = MAX2(err_max_rdiff, ithr_err_max_rdiff);
+                err_max_diff = MAX2(err_max_diff, ithr_err_max_diff);
+            }
+            ithr_zeros = 0;
+            ithr_all_max_rdiff = 0.f;
+            ithr_all_max_diff = 0.f;
+            ithr_err_max_rdiff = 0.f;
+            ithr_err_max_diff = 0.f;
+        }
     };
 
     // parallel comparison to speed up the process
-    benchdnn_parallel_nd(nelems, compare_point_values);
+    // TODO: to speed up the dump process, each thread should prepare its dump
+    // piece in a string object, then the master thread prints them in order.
+    // With this logic, the block of code below won't be needed.
+    benchdnn_parallel_nd(nelems_pad, compare_point_values);
 
     // serial comparison with enabled dumping when needed for nicer output.
-    if (!all_ok || need_dump) {
+    if (!global_ok || need_dump) {
         from_parallel = false;
         for (int64_t i = 0; i < nelems; ++i)
             compare_point_values(i);
@@ -338,17 +496,20 @@ int compare_t::compare_p2p(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
         // and negative in the output equally.
         zero_trust_percent = (100.f + zero_trust_percent_) / 2.f;
     }
-    if (res->state != FAILED && zeros_percent > zero_trust_percent
-            && nelems >= 10)
-        res->state = MISTRUSTED;
+    bool is_mistrusted = zeros_percent > zero_trust_percent && nelems >= 10;
+    if (res->state != FAILED && is_mistrusted) res->state = MISTRUSTED;
 
-    BENCHDNN_PRINT(6, "[COMPARE_STATS]%s: max_diff:%8g max_rdiff:%8g\n",
-            get_kind_str().c_str(), max_diff.load(), max_rdiff.load());
+    // Status may be propagated from previous tensor. Use stats from cur tensor.
+    BENCHDNN_PRINT((n_errors ? 0 : 6),
+            "[COMPARE_STATS]%s: trh=%g err_max_diff:%8g err_max_rdiff:%8g "
+            "all_max_diff:%8g all_max_rdiff:%8g\n",
+            get_kind_str().c_str(), trh_, err_max_diff, err_max_rdiff,
+            all_max_diff, all_max_rdiff);
 
-    BENCHDNN_PRINT((res->state == MISTRUSTED ? 2 : 6),
+    BENCHDNN_PRINT((is_mistrusted ? 2 : 6),
             "[COMPARE_TRUST]%s: z:%2.0f%% (>%2.0f%%) (z: %ld, total: %ld)\n",
             get_kind_str().c_str(), zeros_percent, zero_trust_percent,
-            (long)zeros.load(), (long)nelems);
+            (long)zeros, (long)nelems);
 
     // Set PASSED if no failure in current or previous checks happened and test
     // can be trusted.
@@ -359,11 +520,11 @@ int compare_t::compare_p2p(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
 
 int compare_t::compare(const dnn_mem_t &exp_mem, const dnn_mem_t &got_mem,
         const attr_t &attr, res_t *res) const {
-    std::string add_args = std::string(use_norm_ ? "use_norm:true" : "")
-            + std::string(op_output_has_nans_ ? "has_nans:true" : "");
-    BENCHDNN_PRINT(6, "[COMPARE]%s: trh=%g zero_trust%%=%.2f%% extra=%s\n",
-            get_kind_str().c_str(), trh_, zero_trust_percent_,
-            add_args.c_str());
+    std::string add_args = std::string(use_norm_ ? "use_norm:true;" : "")
+            + std::string(op_output_has_nans_ ? "has_nans:true;" : "")
+            + std::string(has_prim_ref_ ? "has_prim_ref:true;" : "");
+    BENCHDNN_PRINT(6, "[COMPARE]%s: zero_trust%%=%.2f%% extra=%s\n",
+            get_kind_str().c_str(), zero_trust_percent_, add_args.c_str());
     if (use_norm_) return compare_norm(exp_mem, got_mem, attr, res);
     return compare_p2p(exp_mem, got_mem, attr, res);
 }
