@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2023 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
 
 #include "utils/cold_cache.hpp"
 #include "utils/parser.hpp"
+#include "utils/stream_kind.hpp"
 
 #include "dnnl_common.hpp"
 
@@ -142,54 +143,51 @@ attr_t::post_ops_t parse_attr_post_ops_func(const std::string &s) {
                 SAFE_V(FAIL);
             }
         } else if (e.is_convolution_kind()) {
-            if (kind == attr_t::post_ops_t::kind_t::DW) {
-                // `DW` has input of `dw:kXsYpZ`, while rest have `dw_k3sXp1`.
-                // TODO: remove `dw_k3sXp1` version.
-                const auto str_dw_params = get_substr(subs, subs_pos, ':');
-                size_t pos = 0, idx = 0;
+            // `DW` has input of `dw:kXsYpZ`.
+            const auto str_dw_params = get_substr(subs, subs_pos, ':');
+            size_t pos = 0, idx = 0;
 
-                pos += idx;
-                if (str_dw_params[pos] != 'k') {
-                    BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
-                            "Error: depthwise post-op entry",
-                            &str_dw_params[pos], "is not 'k'.");
-                    SAFE_V(FAIL);
-                }
-                // TODO: some safe handling would help here
-                e.convolution.kernel = std::stoi(&str_dw_params[++pos], &idx);
-                if (e.convolution.kernel <= 0) {
-                    BENCHDNN_PRINT(0, "%s\n",
-                            "Error: depthwise post-op kernel must be greater "
-                            "than 0.");
-                    SAFE_V(FAIL);
-                }
-
-                pos += idx;
-                if (str_dw_params[pos] != 's') {
-                    BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
-                            "Error: depthwise post-op entry",
-                            &str_dw_params[pos], "is not 's'.");
-                    SAFE_V(FAIL);
-                }
-                e.convolution.stride = std::stoi(&str_dw_params[++pos], &idx);
-                if (e.convolution.stride <= 0) {
-                    BENCHDNN_PRINT(0, "%s\n",
-                            "Error: depthwise post-op stride must be greater "
-                            "than 0.");
-                    SAFE_V(FAIL);
-                }
-
-                pos += idx;
-                if (str_dw_params[pos] != 'p') {
-                    BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
-                            "Error: depthwise post-op entry",
-                            &str_dw_params[pos], "is not 'p'.");
-                    SAFE_V(FAIL);
-                }
-                e.convolution.padding = std::stoi(&str_dw_params[++pos]);
-
-                if (subs_pos == std::string::npos) continue;
+            pos += idx;
+            if (str_dw_params[pos] != 'k') {
+                BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
+                        "Error: depthwise post-op entry", &str_dw_params[pos],
+                        "is not 'k'.");
+                SAFE_V(FAIL);
             }
+            // TODO: some safe handling would help here
+            e.convolution.kernel = std::stoi(&str_dw_params[++pos], &idx);
+            if (e.convolution.kernel <= 0) {
+                BENCHDNN_PRINT(0, "%s\n",
+                        "Error: depthwise post-op kernel must be greater "
+                        "than 0.");
+                SAFE_V(FAIL);
+            }
+
+            pos += idx;
+            if (str_dw_params[pos] != 's') {
+                BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
+                        "Error: depthwise post-op entry", &str_dw_params[pos],
+                        "is not 's'.");
+                SAFE_V(FAIL);
+            }
+            e.convolution.stride = std::stoi(&str_dw_params[++pos], &idx);
+            if (e.convolution.stride <= 0) {
+                BENCHDNN_PRINT(0, "%s\n",
+                        "Error: depthwise post-op stride must be greater "
+                        "than 0.");
+                SAFE_V(FAIL);
+            }
+
+            pos += idx;
+            if (str_dw_params[pos] != 'p') {
+                BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
+                        "Error: depthwise post-op entry", &str_dw_params[pos],
+                        "is not 'p'.");
+                SAFE_V(FAIL);
+            }
+            e.convolution.padding = std::stoi(&str_dw_params[++pos]);
+
+            if (subs_pos == std::string::npos) continue;
 
             const auto dt_str = get_substr(subs, subs_pos, ':');
             e.convolution.dst_dt = str2dt(dt_str.c_str());
@@ -199,30 +197,14 @@ attr_t::post_ops_t parse_attr_post_ops_func(const std::string &s) {
                         "is not recognized.");
                 SAFE_V(FAIL);
             }
-            if (subs_pos == std::string::npos) continue;
-            CATCH_DANGLING_SYMBOL;
-
-            auto all_scales_str = get_substr(subs, subs_pos, '+');
-            if (e.convolution.wei_scale.from_str(all_scales_str) != OK) {
-                BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
-                        "Error: depthwise post-op weights scale",
-                        all_scales_str.c_str(), "is not recognized.");
+            if (subs_pos < std::string::npos) {
+                const auto leftover = get_substr(subs, subs_pos, '\0');
+                BENCHDNN_PRINT(0,
+                        "Error: no more inputs are expected. Provided: "
+                        "\'%s\'.\n",
+                        leftover.c_str());
                 SAFE_V(FAIL);
             }
-
-            // TODO: is it legit and working at all?
-            size_t dst_scale_pos = 0;
-            for (int i = 0; i < 2; ++i)
-                dst_scale_pos = all_scales_str.find(":", dst_scale_pos + 1);
-            if (dst_scale_pos != std::string::npos) {
-                auto dst_scale_str = all_scales_str.substr(dst_scale_pos + 1);
-                if (e.convolution.dst_scale.from_str(dst_scale_str) != OK) {
-                    BENCHDNN_PRINT(0, "%s \'%s\' %s\n",
-                            "Error: depthwise post-op scale",
-                            dst_scale_str.c_str(), "is not recognized.");
-                    SAFE_V(FAIL);
-                }
-            }
         } else if (e.is_eltwise_kind()) {
             e.eltwise.alpha
                     = parser_utils::stof_safe(get_substr(subs, subs_pos, ':'));
@@ -297,6 +279,114 @@ attr_t::post_ops_t parse_attr_post_ops_func(const std::string &s) {
 
     return v;
 }
+
+attr_t::deterministic_t parse_attr_deterministic_func(const std::string &s) {
+    attr_t::deterministic_t v;
+    if (s.empty()) return v;
+
+    v.enabled = str2bool(s.c_str());
+    return v;
+}
+
+attr_t::fpmath_mode_t parse_attr_fpmath_mode_func(const std::string &s) {
+    attr_t::fpmath_mode_t v;
+    if (s.empty()) return v;
+
+    size_t start_pos = 0;
+    auto subs = get_substr(s, start_pos, ':');
+    v.mode = str2fpmath_mode(subs.c_str());
+    if (start_pos == std::string::npos) return v;
+    if (start_pos >= s.size()) {
+        BENCHDNN_PRINT(0, "%s \'%s\'\n",
+                "Error: dangling symbol at the end of input", s.c_str());
+        SAFE_V(FAIL);
+    }
+
+    if (start_pos != std::string::npos) {
+        subs = get_substr(s, start_pos, '\0');
+        v.apply_to_int = str2bool(subs.c_str());
+
+        if (start_pos != std::string::npos) {
+            BENCHDNN_PRINT(0, "%s \'%s\'\n",
+                    "Error: dangling symbol at the end of input", s.c_str());
+            SAFE_V(FAIL);
+        }
+    }
+
+    return v;
+}
+
+attr_t::rounding_mode_t parse_attr_rounding_mode_func(const std::string &s) {
+    attr_t::rounding_mode_t rm;
+
+    if (s.empty()) return rm;
+
+    size_t start_pos = 0;
+    while (start_pos != std::string::npos) {
+        auto subs = parser::get_substr(s, start_pos, '+');
+        size_t subs_pos = 0;
+
+        auto arg = str2arg(parser::get_substr(subs, subs_pos, ':'));
+        if (arg == DNNL_ARG_UNDEF) {
+            BENCHDNN_PRINT(0, "%s\n", "Error: undefined argument index");
+            SAFE_V(FAIL);
+        }
+        if (subs_pos != std::string::npos)
+            rm.set(arg,
+                    str2rounding_mode(parser::get_substr(subs, subs_pos, ':')));
+        if (subs_pos != std::string::npos)
+            rm.set_seed(stoll_safe(get_substr(subs, subs_pos, ':')));
+    }
+    return rm;
+}
+
+attr_t::dropout_t parse_attr_dropout_func(const std::string &s) {
+    const char *err = "Error: dangling symbol at the end of input";
+    attr_t::dropout_t v;
+    if (s.empty()) return v;
+
+    size_t start_pos = 0;
+    auto subs = get_substr(s, start_pos, ':');
+    v.p = stof_safe(subs);
+    if ((v.p < 0.f) || (v.p > 1.f)) {
+        BENCHDNN_PRINT(0, "Error: bad dropout probability value: %f\n", v.p);
+        SAFE_V(FAIL);
+    }
+    if (start_pos == std::string::npos) return v;
+    if (start_pos >= s.size()) {
+        BENCHDNN_PRINT(0, "%s \'%s\'\n", err, s.c_str());
+        SAFE_V(FAIL);
+    }
+
+    if (start_pos != std::string::npos) {
+        subs = get_substr(s, start_pos, ':');
+        v.seed = stoll_safe(subs);
+
+        if (start_pos == std::string::npos) return v;
+        if (start_pos >= s.size()) {
+            BENCHDNN_PRINT(0, "%s \'%s\'\n", err, s.c_str());
+            SAFE_V(FAIL);
+        }
+
+        if (start_pos != std::string::npos) {
+            v.tag = get_substr(s, start_pos, '\0');
+
+            if (check_tag(v.tag) != OK) {
+                BENCHDNN_PRINT(0, "%s \'%s\' %s\n", "Error: dropout mask tag",
+                        v.tag.c_str(), "is not recognized.");
+                SAFE_V(FAIL);
+            }
+
+            if (start_pos != std::string::npos) {
+                BENCHDNN_PRINT(0, "%s \'%s\'\n", err, s.c_str());
+                SAFE_V(FAIL);
+            }
+        }
+    }
+
+    return v;
+}
+
 } // namespace parser_utils
 
 // vector types
@@ -417,13 +507,12 @@ bool parse_mb(std::vector<int64_t> &mb, const std::vector<int64_t> &def_mb,
 }
 
 bool parse_attr_post_ops(std::vector<attr_t::post_ops_t> &po, const char *str,
-        const std::string &option_name /* = "attr-post-ops"*/) {
+        const std::string &option_name = "attr-post-ops") {
     static const std::string help
             = "POST-OPS\n    Specifies post-ops attribute. `POST-OPS` syntax "
               "is one of those:\n    * SUM[:SCALE[:ZERO_POINT[:DATA_TYPE]]]\n  "
-              "  * ELTWISE[:ALPHA[:BETA[:SCALE]]]\n    * "
-              "DW:KkSsPp[:DST_DT[:WEI_SCALE[:DST_SCALE]]]\n    * "
-              "BINARY:DT[:MASK_INPUT[:TAG]]\n    More details at "
+              "  * ELTWISE[:ALPHA[:BETA[:SCALE]]]\n    * DW:KkSsPp[:DST_DT]\n  "
+              "  * BINARY:DT[:MASK_INPUT[:TAG]]\n    More details at "
               "https://github.com/oneapi-src/oneDNN/blob/master/tests/benchdnn/"
               "doc/knobs_attr.md\n";
     std::vector<attr_t::post_ops_t> def {attr_t::post_ops_t()};
@@ -432,9 +521,9 @@ bool parse_attr_post_ops(std::vector<attr_t::post_ops_t> &po, const char *str,
 }
 
 bool parse_attr_scales(std::vector<attr_t::arg_scales_t> &scales,
-        const char *str, const std::string &option_name /* = "attr-scales"*/) {
+        const char *str, const std::string &option_name = "attr-scales") {
     static const std::string help
-            = "ARG:POLICY[:SCALE[*]][+...]\n    Specifies input scales "
+            = "ARG:POLICY[:SCALE][+...]\n    Specifies input scales "
               "attribute.\n    More details at "
               "https://github.com/oneapi-src/oneDNN/blob/master/tests/benchdnn/"
               "doc/knobs_attr.md\n";
@@ -442,21 +531,33 @@ bool parse_attr_scales(std::vector<attr_t::arg_scales_t> &scales,
 }
 
 bool parse_attr_zero_points(std::vector<attr_t::zero_points_t> &zp,
-        const char *str,
-        const std::string &option_name /* = "attr-zero-points"*/) {
+        const char *str, const std::string &option_name = "attr-zero-points") {
     static const std::string help
-            = "ARG:POLICY:ZEROPOINT[*][+...]\n    Specifies zero-points "
+            = "ARG:POLICY[:ZEROPOINT][+...]\n    Specifies zero-points "
               "attribute.\n    More details at "
               "https://github.com/oneapi-src/oneDNN/blob/master/tests/benchdnn/"
               "doc/knobs_attr.md\n";
     return parse_subattr(zp, str, option_name, help);
 }
 
+bool parse_attr_rounding_mode(std::vector<attr_t::rounding_mode_t> &rm,
+        const char *str,
+        const std::string &option_name = "attr-rounding-mode") {
+    static const std::string help
+            = "ARG:MODE[:SEED][+...]    (Default: `environment`)\n    "
+              "Specifies a "
+              "rounding mode MODE to be applied upon conversion of argument "
+              "ARG.\n    More details at "
+            + doc_url + "knobs_attr.md\n";
+    return parse_vector_option(rm, {},
+            parser_utils::parse_attr_rounding_mode_func, str, option_name,
+            help);
+}
+
 bool parse_attr_scratchpad_mode(
         std::vector<dnnl_scratchpad_mode_t> &scratchpad_mode,
         const std::vector<dnnl_scratchpad_mode_t> &def_scratchpad_mode,
-        const char *str,
-        const std::string &option_name /* = "attr-scratchpad"*/) {
+        const char *str, const std::string &option_name = "attr-scratchpad") {
     static const std::string help
             = "MODE    (Default: `library`)\n    Specifies scratchpad "
               "attribute. `MODE` values can be `library` or `user`.\n    More "
@@ -466,16 +567,68 @@ bool parse_attr_scratchpad_mode(
             str2scratchpad_mode, str, option_name, help);
 }
 
-bool parse_attr_fpmath_mode(std::vector<dnnl_fpmath_mode_t> &fpmath_mode,
-        const std::vector<dnnl_fpmath_mode_t> &def_fpmath_mode, const char *str,
-        const std::string &option_name /* = "attr-fpmath"*/) {
+bool parse_attr_fpmath_mode(std::vector<attr_t::fpmath_mode_t> &fpmath_mode,
+        const std::vector<attr_t::fpmath_mode_t> &def_fpmath_mode,
+        const char *str, const std::string &option_name = "attr-fpmath") {
     static const std::string help
-            = "MODE    (Default: `strict`)\n    Specifies fpmath_mode "
-              "attribute. `MODE` values can be `strict` or `bf16`.\n";
-    return parse_vector_option(fpmath_mode, def_fpmath_mode, str2fpmath_mode,
+            = "MODE[:APPLY_TO_INT]    (Default: `strict[:false]`)\n    "
+              "Specifies "
+              "fpmath_mode attribute. `MODE` values can be `strict` or "
+              "`bf16`. `APPLY_TO_INT` values can be `true` or `false`.\n";
+    return parse_vector_option(fpmath_mode, def_fpmath_mode,
+            parser_utils::parse_attr_fpmath_mode_func, str, option_name, help);
+}
+
+bool parse_attr_dropout(std::vector<attr_t::dropout_t> &dropout,
+        const std::vector<attr_t::dropout_t> &def_dropout, const char *str,
+        const std::string &option_name = "attr-dropout") {
+    static const std::string help
+            = "PROBABILITY[:SEED[:TAG]]\n    Specifies dropout attribute.\n    "
+              "More details at "
+            + doc_url + "knobs_attr.md\n";
+    return parse_vector_option(dropout, def_dropout,
+            parser_utils::parse_attr_dropout_func, str, option_name, help);
+}
+
+bool parse_attr_acc_mode(std::vector<dnnl_accumulation_mode_t> &acc_mode,
+        const std::vector<dnnl_accumulation_mode_t> &def_acc_mode,
+        const char *str, const std::string &option_name = "attr-acc-mode") {
+    static const std::string help
+            = "MODE    (Default: `strict`)\n    Specifies accumulation mode "
+              "attribute. `MODE` values can be `strict`, `relaxed`, `any`,"
+              "`f32`, `f16` or `s32`.\n";
+    return parse_vector_option(acc_mode, def_acc_mode, str2accumulation_mode,
             str, option_name, help);
 }
 
+bool parse_attr_deterministic(
+        std::vector<attr_t::deterministic_t> &deterministic,
+        const std::vector<attr_t::deterministic_t> &def_deterministic,
+        const char *str,
+        const std::string &option_name = "attr-deterministic") {
+    static const std::string help
+            = "MODE    (Default: `false`)\n    Specifies deterministic mode "
+              "attribute. `MODE` values can be `true`, or `false`.\n";
+    return parse_vector_option(deterministic, def_deterministic,
+            parser_utils::parse_attr_deterministic_func, str, option_name,
+            help);
+}
+
+bool parse_attributes(
+        base_settings_t &s, const base_settings_t &def, const char *str) {
+    const bool parsed_attrs = parse_attr_scales(s.scales, str)
+            || parse_attr_zero_points(s.zero_points, str)
+            || parse_attr_post_ops(s.post_ops, str)
+            || parse_attr_dropout(s.dropout, def.dropout, str)
+            || parse_attr_scratchpad_mode(
+                    s.scratchpad_mode, def.scratchpad_mode, str)
+            || parse_attr_fpmath_mode(s.fpmath_mode, def.fpmath_mode, str)
+            || parse_attr_acc_mode(s.acc_mode, def.acc_mode, str)
+            || parse_attr_deterministic(s.deterministic, def.deterministic, str)
+            || parse_attr_rounding_mode(s.rounding_mode, str);
+    return parsed_attrs;
+}
+
 bool parse_axis(std::vector<int> &axis, const std::vector<int> &def_axis,
         const char *str, const std::string &option_name /* = "axis"*/) {
     static const std::string help
@@ -523,11 +676,11 @@ bool parse_strides(std::vector<vdims_t> &strides,
         const std::vector<vdims_t> &def_strides, const char *str,
         const std::string &option_name /* = "strides"*/) {
     static const std::string help
-            = "DIMS_SRC:DIMS_WEI:DIMS_DST    (Default: not specified)\n    "
-              "Specifies strides `DIMS_ARG` for correspondent `ARG`.\n    If "
-              "correspondent `DIMS_ARG` is empty, it does not take an "
-              "effect.\n    More details at "
-            + doc_url + "driver_matmul.md\n";
+            = "DIMS_SRC[:DIMS_WEI]:DIMS_DST    (Default: not specified)\n    "
+              "Specifies strides `DIMS_ARG` for correspondent supported "
+              "`ARG`.\n    If correspondent `DIMS_ARG` is empty, it does not "
+              "take an effect.\n    More details at "
+            + doc_url + "driver_" + driver_name + ".md\n";
     auto str2strides = [&](const char *str) -> vdims_t {
         vdims_t strides(STRIDES_SIZE);
         parse_multivector_str(
@@ -706,6 +859,17 @@ static bool parse_canonical(
             canonical, false, str2bool, str, option_name, help);
 }
 
+static bool parse_check_ref_impl(
+        const char *str, const std::string &option_name = "check-ref-impl") {
+    static const std::string help
+            = "BOOL    (Default: `false`)\n    Instructs the driver to compare "
+              "an implementation name against the \'ref\' string pattern.\n    "
+              "When set to `true`, the check would return an error if the "
+              "implementation name contains such pattern.\n";
+    return parse_single_value_option(
+            check_ref_impl, false, str2bool, str, option_name, help);
+}
+
 static bool parse_cold_cache(
         const char *str, const std::string &option_name = "cold-cache") {
     static const std::string help
@@ -779,20 +943,45 @@ static bool parse_engine(
     return true;
 }
 
-static bool parse_fast_ref_gpu(
-        const char *str, const std::string &option_name = "fast-ref-gpu") {
+static bool parse_fast_ref(
+        const char *str, const std::string &option_name = "fast-ref") {
     static const std::string help
             = "BOOL    (Default: `true`)\n    Instructs the driver to use "
               "faster reference path when doing correctness testing for "
               "`--engine=gpu`.\n    When set to `true`, the library best fit "
               "CPU implementation is used to compute the reference path.\n";
     bool parsed = parse_single_value_option(
-            fast_ref_gpu, true, str2bool, str, option_name, help);
+            fast_ref, default_fast_ref, str2bool, str, option_name, help);
 #if DNNL_CPU_RUNTIME == DNNL_RUNTIME_NONE
-    if (parsed && fast_ref_gpu) {
-        fast_ref_gpu = false;
+    if (parsed && fast_ref) {
+        fast_ref = false;
         fprintf(stderr,
-                "%s driver: WARNING: option `fast_ref_gpu` is not supported "
+                "%s driver: WARNING: option `fast_ref` is not supported "
+                "for GPU only configurations.\n",
+                driver_name.c_str());
+    }
+#endif
+    return parsed;
+}
+
+// TODO: remove
+static bool parse_fast_ref_gpu(
+        const char *str, const std::string &option_name = "fast-ref-gpu") {
+    bool parsed = parse_single_value_option(fast_ref, default_fast_ref,
+            str2bool, str, option_name, std::string());
+
+    static bool msg_printed = false;
+    if (parsed && !msg_printed) {
+        BENCHDNN_PRINT(0, "%s\n",
+                "Warning: \'--fast-ref-gpu\' is deprecated. Use \'--fast-ref\' "
+                "instead.");
+        msg_printed = true;
+    }
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_NONE
+    if (parsed && fast_ref) {
+        fast_ref = false;
+        fprintf(stderr,
+                "%s driver: WARNING: option `fast_ref` is not supported "
                 "for GPU only configurations.\n",
                 driver_name.c_str());
     }
@@ -890,6 +1079,24 @@ static bool parse_max_ms_per_prb(
     return parsed;
 }
 
+static bool parse_num_streams(
+        const char *str, const std::string &option_name = "num-streams") {
+    static const std::string help
+            = "N    (Default: `1`)\n    Specifies the number `N` of streams "
+              "used for performance benchmarking.\n    `N` is a positive "
+              "integer.\n";
+    bool parsed = parse_single_value_option(num_streams, default_num_streams,
+            parser_utils::stoll_safe, str, option_name, help);
+    if (parsed) {
+        if (num_streams <= 0) {
+            BENCHDNN_PRINT(
+                    0, "%s\n", "Error: number of streams must be positive.");
+            SAFE_V(FAIL);
+        }
+    }
+    return parsed;
+}
+
 static bool parse_repeats_per_prb(
         const char *str, const std::string &option_name = "repeats-per-prb") {
     static const std::string help
@@ -946,6 +1153,7 @@ static bool parse_mode(
               "    * `C` for correctness testing.\n"
               "    * `P` for performance testing.\n"
               "    * `F` for fast performance testing (GPU only).\n"
+              "    * `B` for bitwise (numerical determinism) testing.\n"
               "    * `CP` for both correctness and performance testing.\n"
               "    More details at "
             + doc_url + "benchdnn_general_info.md\n";
@@ -955,7 +1163,7 @@ static bool parse_mode(
         if (_str.size() > 2) {
             BENCHDNN_PRINT(
                     0, "%s\n%s", "Error: mode value is invalid.", help.c_str());
-            exit(2);
+            SAFE_V(FAIL);
         } else if (_str.size() == 2) {
             for (size_t i = 0; i < _str.size(); i++) {
                 switch (_str[i]) {
@@ -966,7 +1174,7 @@ static bool parse_mode(
                     default:
                         BENCHDNN_PRINT(0, "%s\n%s",
                                 "Error: mode value is invalid.", help.c_str());
-                        exit(2);
+                        SAFE_V(FAIL);
                 }
             }
             mode = bench_mode_t::corr_perf;
@@ -989,10 +1197,12 @@ static bool parse_mode(
                     bench_mode_modifier = mode_modifier_t::par_create
                             | mode_modifier_t::no_host_memory;
                     break;
+                case 'b':
+                case 'B': mode = bench_mode_t::bitwise; break;
                 default:
                     BENCHDNN_PRINT(0, "%s\n%s", "Error: mode value is invalid.",
                             help.c_str());
-                    exit(2);
+                    SAFE_V(FAIL);
             }
         }
 
@@ -1077,6 +1287,28 @@ static bool parse_start(
             test_start, 0, parser_utils::stoll_safe, str, option_name, help);
 }
 
+static bool parse_stream_kind(
+        const char *str, const std::string &option_name = "stream-kind") {
+    static const std::string help
+            = "KIND    (Default: `def`)\n    Specifies a stream `KIND` to test "
+              "with DPC++ and OpenCL engines through stream flags.\n    "
+              "`KIND` values are `def` (the default flags), `in_order`, or "
+              "`out_of_order`.\n";
+    bool parsed = parse_single_value_option(stream_kind, default_stream_kind,
+            str2stream_kind, str, option_name, help);
+
+#if !defined(DNNL_WITH_SYCL) && DNNL_GPU_RUNTIME != DNNL_RUNTIME_OCL
+    if (parsed) {
+        BENCHDNN_PRINT(0,
+                "Error: option `--%s` is supported with DPC++ and OpenCL "
+                "builds only, exiting...\n",
+                option_name.c_str());
+        SAFE_V(FAIL);
+    }
+#endif
+    return parsed;
+}
+
 static bool parse_verbose(
         const char *str, const std::string &option_name = "verbose") {
     static const std::string help
@@ -1112,13 +1344,14 @@ bool parse_bench_settings(const char *str) {
 
     bool parsed = parse_allow_enum_tags_only(str)
             || parse_attr_same_pd_check(str) || parse_canonical(str)
-            || parse_cold_cache(str) || parse_cpu_isa_hints(str)
-            || parse_engine(str) || parse_fast_ref_gpu(str)
+            || parse_check_ref_impl(str) || parse_cold_cache(str)
+            || parse_cpu_isa_hints(str) || parse_engine(str)
+            || parse_fast_ref(str) || parse_fast_ref_gpu(str)
             || parse_fix_times_per_prb(str) || parse_max_ms_per_prb(str)
-            || parse_repeats_per_prb(str) || parse_mem_check(str)
-            || parse_memory_kind(str) || parse_mode(str)
+            || parse_num_streams(str) || parse_repeats_per_prb(str)
+            || parse_mem_check(str) || parse_memory_kind(str) || parse_mode(str)
             || parse_mode_modifier(str) || parse_skip_impl(str)
-            || parse_start(str) || parse_verbose(str);
+            || parse_start(str) || parse_stream_kind(str) || parse_verbose(str);
 
     // Last condition makes this help message to be triggered once driver_name
     // is already known.
